from PyQt4 import QtGui, QtCore
from nupic_studio.htm.network import Network
from nupic_studio.htm.node import NodeType, Node
from nupic_studio.htm.node_region import Region
from nupic_studio.htm.node_sensor import Sensor, DataSourceType, PredictionsMethod
from nupic_studio.htm.encoding import Encoding
from nupic_studio.htm.link import Link

"""
Loads and saves the Elements of the .nuproj file, that contains user entries for project properties
Provides loaded elements as a structure to return.
"""

class Project:
  """
  Loads and saves the Elements of the Project file, that contains user entries for Network configuration
  Provides loaded elements as a structure to return.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.fileName = ''
    """Project file"""

    self.name = "Untitled"
    """Name of the project."""

    self.author = ""
    """Author of the project."""

    self.description = ""
    """Description of the project."""

    self.network = Network()
    """The network created for the project."""

    #endregion

  #endregion

  #region Methods

  def new(self):
    """
    Initializes a new instance of this class.
    """

    # Initialize metadata
    self.fileName = ''
    self.name = "Untitled"
    self.author = ""
    self.description = ""

    # Create the top region
    topRegion = Region("TopRegion")

    # Create the network and add topRegion as its starting node
    self.network = Network()
    self.network.nodes.append(topRegion)
    self.network.preparePhases()

  def open(self, fileName):
    """
    Loads the content from XML file to Project instance.
    """

    # Create the network
    self.network = Network()

    self.fileName = fileName
    file = QtCore.QFile(self.fileName)
    if (file.open(QtCore.QIODevice.ReadOnly)):
      xmlReader = QtCore.QXmlStreamReader()
      xmlReader.setDevice(file)
      while (not xmlReader.isEndDocument()):
        if xmlReader.isStartElement():
          if xmlReader.name().toString() == 'MetaData':
            self.name = self.__getStringAttribute(xmlReader.attributes(), 'name')
            self.author = self.__getStringAttribute(xmlReader.attributes(), 'author')
            self.description = self.__getStringAttribute(xmlReader.attributes(), 'description')
          elif xmlReader.name().toString() == 'Net':
            while xmlReader.readNextStartElement():
              if xmlReader.name().toString() == 'Node':
                node = self.__readNode(xmlReader)
                self.network.nodes.append(node)
              elif xmlReader.name().toString() == 'Link':
                link = self.__readLink(xmlReader)
                self.network.links.append(link)

        xmlReader.readNext()
      if (xmlReader.hasError()):
        QtGui.QMessageBox.critical(self, "Critical", "Ocurred a XML error: " + xmlReader.errorString().data(), QtGui.QMessageBox.Ok | QtGui.QMessageBox.Default, QtGui.QMessageBox.NoButton)
    else:
      QtGui.QMessageBox.critical(self, "Critical", "Cannot read the project file!", QtGui.QMessageBox.Ok | QtGui.QMessageBox.Default, QtGui.QMessageBox.NoButton)

    self.network.preparePhases()

  def __getStringAttribute(self, attributes, attributeName):
    if attributes.value(attributeName).toString() != "":
      attributeValue = str(attributes.value(attributeName).toString())
    else:
      attributeValue = ""
    return attributeValue

  def __getIntegerAttribute(self, attributes, attributeName):
    attributeValue = 0
    if attributes.value(attributeName).toString() != "":
      attributeValue = int(attributes.value(attributeName).toString())
    return attributeValue

  def __getFloatAttribute(self, attributes, attributeName):
    attributeValue = 0.0
    if attributes.value(attributeName).toString() != "":
      attributeValue = float(attributes.value(attributeName).toString())
    return attributeValue

  def __getBooleanAttribute(self, attributes, attributeName):
    attributeValue = False
    if attributes.value(attributeName).toString() == "True":
      attributeValue = True
    return attributeValue

  def __readNode(self, xmlReader):

    # Read type of node
    name = self.__getStringAttribute(xmlReader.attributes(), 'name')
    type = self.__getStringAttribute(xmlReader.attributes(), 'type')

    # Create a node from parameters
    node = None
    if type == 'Region':
      node = Region(name)
    elif type == 'Sensor':
      node = Sensor(name)
    node.width = self.__getIntegerAttribute(xmlReader.attributes(), 'width')
    node.height = self.__getIntegerAttribute(xmlReader.attributes(), 'height')

    # Read specific parameters according to node type
    if type == 'Region':
      node.enableSpatialLearning = self.__getBooleanAttribute(xmlReader.attributes(), 'enableSpatialLearning')
      node.potentialRadius = self.__getIntegerAttribute(xmlReader.attributes(), 'potentialRadius')
      node.potentialPct = self.__getFloatAttribute(xmlReader.attributes(), 'potentialPct')
      node.globalInhibition = self.__getBooleanAttribute(xmlReader.attributes(), 'globalInhibition')
      node.localAreaDensity = self.__getFloatAttribute(xmlReader.attributes(), 'localAreaDensity')
      node.numActiveColumnsPerInhArea = self.__getFloatAttribute(xmlReader.attributes(), 'numActiveColumnsPerInhArea')
      node.stimulusThreshold = self.__getIntegerAttribute(xmlReader.attributes(), 'stimulusThreshold')
      node.proximalSynConnectedPerm = self.__getFloatAttribute(xmlReader.attributes(), 'proximalSynConnectedPerm')
      node.proximalSynPermIncrement = self.__getFloatAttribute(xmlReader.attributes(), 'proximalSynPermIncrement')
      node.proximalSynPermDecrement = self.__getFloatAttribute(xmlReader.attributes(), 'proximalSynPermDecrement')
      node.minPctOverlapDutyCycle = self.__getFloatAttribute(xmlReader.attributes(), 'minPctOverlapDutyCycle')
      node.minPctActiveDutyCycle = self.__getFloatAttribute(xmlReader.attributes(), 'minPctActiveDutyCycle')
      node.dutyCyclePeriod = self.__getIntegerAttribute(xmlReader.attributes(), 'dutyCyclePeriod')
      node.maxBoost = self.__getFloatAttribute(xmlReader.attributes(), 'maxBoost')
      node.spSeed = self.__getIntegerAttribute(xmlReader.attributes(), 'spSeed')
      node.enableTemporalLearning = self.__getBooleanAttribute(xmlReader.attributes(), 'enableTemporalLearning')
      node.numCellsPerColumn = self.__getIntegerAttribute(xmlReader.attributes(), 'numCellsPerColumn')
      node.distalSynInitialPerm = self.__getFloatAttribute(xmlReader.attributes(), 'distalSynInitialPerm')
      node.distalSynConnectedPerm = self.__getFloatAttribute(xmlReader.attributes(), 'distalSynConnectedPerm')
      node.distalSynPermIncrement = self.__getFloatAttribute(xmlReader.attributes(), 'distalSynPermIncrement')
      node.distalSynPermDecrement = self.__getFloatAttribute(xmlReader.attributes(), 'distalSynPermDecrement')
      node.minThreshold = self.__getIntegerAttribute(xmlReader.attributes(), 'minThreshold')
      node.activationThreshold = self.__getIntegerAttribute(xmlReader.attributes(), 'activationThreshold')
      node.maxNumNewSynapses = self.__getIntegerAttribute(xmlReader.attributes(), 'maxNumNewSynapses')
      node.tpSeed = self.__getIntegerAttribute(xmlReader.attributes(), 'tpSeed')
    elif type == 'Sensor':
      dataSourceType = self.__getStringAttribute(xmlReader.attributes(), 'dataSourceType')
      if dataSourceType == "File":
        node.dataSourceType = DataSourceType.file
        node.fileName = self.__getStringAttribute(xmlReader.attributes(), 'fileName')
      elif dataSourceType == "Database":
        node.dataSourceType = DataSourceType.database
        node.databaseConnectionString = self.__getStringAttribute(xmlReader.attributes(), 'databaseConnectionString')
        node.databaseTable = self.__getStringAttribute(xmlReader.attributes(), 'databaseTable')
      node.predictionsMethod = self.__getStringAttribute(xmlReader.attributes(), 'predictionsMethod')
      if node.predictionsMethod == PredictionsMethod.classification:
        node.enableClassificationLearning = self.__getBooleanAttribute(xmlReader.attributes(), 'enableClassificationLearning')
        node.enableClassificationInference = self.__getBooleanAttribute(xmlReader.attributes(), 'enableClassificationInference')

      # If still is not end of element it's because this node has encodings
      token = xmlReader.readNext()
      if not xmlReader.isEndElement():
        while xmlReader.readNextStartElement():
          encoding = self.__readEncoding(xmlReader)
          node.encodings.append(encoding)

    token = xmlReader.readNext()

    return node

  def __readEncoding(self, xmlReader):

    # Create a encoding from parameters
    encoding = Encoding()
    encoding.dataSourceFieldName = self.__getStringAttribute(xmlReader.attributes(), 'dataSourceFieldName')
    encoding.dataSourceFieldDataType = self.__getStringAttribute(xmlReader.attributes(), 'dataSourceFieldDataType')
    encoding.enableInference = self.__getBooleanAttribute(xmlReader.attributes(), 'enableInference')
    encoding.encoderModule = self.__getStringAttribute(xmlReader.attributes(), 'encoderModule')
    encoding.encoderClass = self.__getStringAttribute(xmlReader.attributes(), 'encoderClass')
    encoding.encoderParams = self.__getStringAttribute(xmlReader.attributes(), 'encoderParams')
    encoding.encoderFieldName = self.__getStringAttribute(xmlReader.attributes(), 'encoderFieldName')
    encoding.encoderFieldDataType = self.__getStringAttribute(xmlReader.attributes(), 'encoderFieldDataType')
    token = xmlReader.readNext()

    return encoding

  def __readLink(self, xmlReader):

    # Read link parameters
    outNodeName = self.__getStringAttribute(xmlReader.attributes(), 'outNode')
    inNodeName = self.__getStringAttribute(xmlReader.attributes(), 'inNode')
    token = xmlReader.readNext()

    # Find output node instance
    outNode = None
    for node in self.network.nodes:
      if node.name == outNodeName:
        outNode = node
        break

    # Find input node instance
    inNode = None
    for node in self.network.nodes:
      if node.name == inNodeName:
        inNode = node
        break

    # Create a link from parameters
    link = Link()
    link.outNode = outNode
    link.inNode = inNode

    return link

  def save(self, fileName):
    """
    Saves the content from Project instance to XML file.
    """

    self.fileName = fileName
    file = QtCore.QFile(self.fileName)
    file.open(QtCore.QIODevice.WriteOnly)
    xmlWriter = QtCore.QXmlStreamWriter(file)
    xmlWriter.setAutoFormatting(True)
    xmlWriter.writeStartDocument()
    xmlWriter.writeStartElement('Project')

    xmlWriter.writeStartElement('MetaData')
    xmlWriter.writeAttribute('name', self.name)
    xmlWriter.writeAttribute('author', self.author)
    xmlWriter.writeAttribute('description', self.description)
    xmlWriter.writeEndElement()

    xmlWriter.writeStartElement('Net')
    for node in self.network.nodes:
      self.__writeNode(node, xmlWriter)
    for link in self.network.links:
      self.__writeLink(link, xmlWriter)
    xmlWriter.writeEndElement()

    xmlWriter.writeEndElement()
    xmlWriter.writeEndDocument()
    file.close()

  def __writeNode(self, node, xmlWriter):

    # Write common parameters
    xmlWriter.writeStartElement('Node')
    xmlWriter.writeAttribute('name', node.name)

    # Write specific parameters according to node type
    if node.type == NodeType.region:
      xmlWriter.writeAttribute('type', 'Region')
      xmlWriter.writeAttribute('width', str(node.width))
      xmlWriter.writeAttribute('height', str(node.height))
      xmlWriter.writeAttribute('enableSpatialLearning', str(node.enableSpatialLearning))
      xmlWriter.writeAttribute('potentialRadius', str(node.potentialRadius))
      xmlWriter.writeAttribute('potentialPct', str(node.potentialPct))
      xmlWriter.writeAttribute('globalInhibition', str(node.globalInhibition))
      xmlWriter.writeAttribute('localAreaDensity', str(node.localAreaDensity))
      xmlWriter.writeAttribute('numActiveColumnsPerInhArea', str(node.numActiveColumnsPerInhArea))
      xmlWriter.writeAttribute('stimulusThreshold', str(node.stimulusThreshold))
      xmlWriter.writeAttribute('proximalSynConnectedPerm', str(node.proximalSynConnectedPerm))
      xmlWriter.writeAttribute('proximalSynPermIncrement', str(node.proximalSynPermIncrement))
      xmlWriter.writeAttribute('proximalSynPermDecrement', str(node.proximalSynPermDecrement))
      xmlWriter.writeAttribute('minPctOverlapDutyCycle', str(node.minPctOverlapDutyCycle))
      xmlWriter.writeAttribute('minPctActiveDutyCycle', str(node.minPctActiveDutyCycle))
      xmlWriter.writeAttribute('dutyCyclePeriod', str(node.dutyCyclePeriod))
      xmlWriter.writeAttribute('maxBoost', str(node.maxBoost))
      xmlWriter.writeAttribute('spSeed', str(node.spSeed))
      xmlWriter.writeAttribute('enableTemporalLearning', str(node.enableTemporalLearning))
      xmlWriter.writeAttribute('numCellsPerColumn', str(node.numCellsPerColumn))
      xmlWriter.writeAttribute('distalSynInitialPerm', str(node.distalSynInitialPerm))
      xmlWriter.writeAttribute('distalSynConnectedPerm', str(node.distalSynConnectedPerm))
      xmlWriter.writeAttribute('distalSynPermIncrement', str(node.distalSynPermIncrement))
      xmlWriter.writeAttribute('distalSynPermDecrement', str(node.distalSynPermDecrement))
      xmlWriter.writeAttribute('minThreshold', str(node.minThreshold))
      xmlWriter.writeAttribute('activationThreshold', str(node.activationThreshold))
      xmlWriter.writeAttribute('maxNumNewSynapses', str(node.maxNumNewSynapses))
      xmlWriter.writeAttribute('tpSeed', str(node.tpSeed))
    elif node.type == NodeType.sensor:
      xmlWriter.writeAttribute('type', 'Sensor')
      xmlWriter.writeAttribute('width', str(node.width))
      xmlWriter.writeAttribute('height', str(node.height))
      if node.dataSourceType == DataSourceType.file:
        xmlWriter.writeAttribute('dataSourceType', "File")
        xmlWriter.writeAttribute('fileName', node.fileName)
      elif node.dataSourceType == DataSourceType.database:
        xmlWriter.writeAttribute('dataSourceType', "Database")
        xmlWriter.writeAttribute('databaseConnectionString', node.databaseConnectionString)
        xmlWriter.writeAttribute('databaseTable', node.databaseTable)
      xmlWriter.writeAttribute('predictionsMethod', node.predictionsMethod)
      if node.predictionsMethod == PredictionsMethod.classification:
        xmlWriter.writeAttribute('enableClassificationLearning', str(node.enableClassificationLearning))
        xmlWriter.writeAttribute('enableClassificationInference', str(node.enableClassificationInference))

      # Tranverse all encodings
      for encoding in node.encodings:
        self.__writeEncoding(encoding, xmlWriter)

    xmlWriter.writeEndElement()

  def __writeEncoding(self, encoding, xmlWriter):

    # Write encoding parameters
    xmlWriter.writeStartElement('Encoding')
    xmlWriter.writeAttribute('dataSourceFieldName', encoding.dataSourceFieldName)
    xmlWriter.writeAttribute('dataSourceFieldDataType', encoding.dataSourceFieldDataType)
    xmlWriter.writeAttribute('enableInference', str(encoding.enableInference))
    xmlWriter.writeAttribute('encoderModule', encoding.encoderModule)
    xmlWriter.writeAttribute('encoderClass', encoding.encoderClass)
    xmlWriter.writeAttribute('encoderParams', encoding.encoderParams)
    xmlWriter.writeAttribute('encoderFieldName', encoding.encoderFieldName)
    xmlWriter.writeAttribute('encoderFieldDataType', encoding.encoderFieldDataType)
    xmlWriter.writeEndElement()

  def __writeLink(self, link, xmlWriter):

    # Write encoding parameters
    xmlWriter.writeStartElement('Link')
    xmlWriter.writeAttribute('outNode', link.outNode.name)
    xmlWriter.writeAttribute('inNode', link.inNode.name)
    xmlWriter.writeEndElement()

  #endregion




class Link:
  """
  A class only to group properties related to links among nodes in hierarchy.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.outNode = None
    """Node which its output will be used as input to the second node."""

    self.inNode = None
    """Target region to receive the input."""

    #endregion

  #endregion



from nupic_studio import MachineState
from nupic_studio.htm import maxPreviousSteps
from nupic_studio.ui import Global

class Synapse:
  """
  A class only to group properties related to synapses.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.indexSP = -1
    """Index of this cell in the spatial pooler."""

    self.indexTP = -1
    """Index of this synapse in the temporal pooler."""

    self.inputElem = None
    """An input element is a cell in case of the source be a column or then a bit in case of the source be a sensor"""

    self.permanence = MachineState(0., maxPreviousSteps)
    """Permanence of this synapse."""

    # States of this element
    self.isConnected = MachineState(False, maxPreviousSteps)
    self.isPredicted = MachineState(False, maxPreviousSteps)
    self.isFalselyPredicted = MachineState(False, maxPreviousSteps)
    self.isRemoved = MachineState(False, maxPreviousSteps)

    #region Statistics properties

    self.statsConnectionCount = 0
    self.statsConnectionRate = 0.
    self.statsPreditionCount = 0
    self.statsPrecisionRate = 0.

    #endregion

    #region 3d-tree properties (simulation form)

    self.tree3d_initialized = False
    self.tree3d_item = None
    self.tree3d_selected = False

    #endregion

    #endregion

  #endregion

  #region Methods

  def nextStep(self):
    """
    Perfoms actions related to time step progression.
    """

    # Update states machine by remove the first element and add a new element in the end
    self.permanence.rotate()
    self.isConnected.rotate()
    self.isPredicted.rotate()
    self.isFalselyPredicted.rotate()
    self.isRemoved.rotate()

  def calculateStatistics(self):
    """
    Calculate statistics after an iteration.
    """

    # Calculate statistics
    if self.isConnected.atCurrStep():
      self.statsConnectionCount += 1
    if self.isPredicted.atCurrStep():
      self.statsPreditionCount += 1
    if Global.currStep > 0:
      self.statsConnectionRate = self.statsConnectionCount / float(Global.currStep)
    if self.statsConnectionCount > 0:
      self.statsPrecisionRate = self.statsPreditionCount / float(self.statsConnectionCount)

  #endregion



import os
import collections
import numpy
import operator
import math
import dateutil.parser
from PyQt4 import QtGui, QtCore
from nupic_studio import getInstantiatedClass
from nupic_studio.ui import Global
from nupic_studio.htm import maxPreviousSteps, maxFutureSteps, maxPreviousStepsWithInference
from nupic_studio.htm.node import Node, NodeType
from nupic_studio.htm.bit import Bit
from nupic_studio.htm.encoding import FieldDataType
from nupic.encoders import MultiEncoder
from nupic.data.file_record_stream import FileRecordStream

class DataSourceType:
  """
  Types of data sources which a sensor gets inputs.
  """

  file = 1
  database = 2

class PredictionsMethod:
  """
  Methods used to get predicted values and their probabilities
  """

  reconstruction = "Reconstruction"
  classification = "Classification"

class Sensor(Node):
  """
  A super class only to group properties related to sensors.
  """

  #region Constructor

  def __init__(self, name):
    """
    Initializes a new instance of this class.
    """

    Node.__init__(self, name, NodeType.sensor)

    #region Instance fields

    self.bits = []
    """An array of the bit objects that compose the current output of this node."""

    self.dataSource = None
    """Data source which provides records to fed into a region."""

    self.dataSourceType = DataSourceType.file
    """Type of the data source (File or Database)"""

    self.fileName = ''
    """The input file name to be handled. Returns the input file name only if it is in the project directory, full path otherwise."""

    self.databaseConnectionString = ""
    """Connection string of the database."""

    self.databaseTable = ''
    """Target table of the database."""

    self.encoder = None
    """Multi-encoder which concatenate sub-encodings to convert raw data to htm input and vice-versa."""

    self.encodings = []
    """List of sub-encodings that handles the input from database"""

    self.predictionsMethod = PredictionsMethod.reconstruction
    """Method used to get predicted values and their probabilities."""

    self.enableClassificationLearning = True
    """Switch for classification learning"""

    self.enableClassificationInference = True
    """Switch for classification inference"""

    #endregion

    #region Statistics properties

    self.statsPrecisionRate = 0.

    #endregion

  #endregion

  #region Methods

  def getBit(self, x, y):
    """
    Return the bit located at given position
    """

    bit = self.bits[(y * self.width) + x]

    return bit

  def initialize(self):
    """
    Initialize this node.
    """

    Node.initialize(self)

    # Initialize input bits
    self.bits = []
    for x in range(self.width):
      for y in range(self.height):
        bit = Bit()
        bit.x = x
        bit.y = y
        self.bits.append(bit)

    if self.dataSourceType == DataSourceType.file:
      """
      Initialize this node opening the file and place cursor on the first record.
      """

      # If file name provided is a relative path, use project file path
      if self.fileName != '' and os.path.dirname(self.fileName) == '':
        fullFileName = os.path.dirname(Global.project.fileName) + '/' + self.fileName
      else:
        fullFileName = self.fileName

      # Check if file really exists
      if not os.path.isfile(fullFileName):
        QtGui.QMessageBox.warning(None, "Warning", "Input stream file '" + fullFileName + "' was not found or specified.", QtGui.QMessageBox.Ok)
        return

      # Create a data source for read the file
      self.dataSource = FileRecordStream(fullFileName)

    elif self.dataSourceType == DataSourceType.database:
      pass

    self.encoder = MultiEncoder()
    for encoding in self.encodings:
      encoding.initialize()

      # Create an instance class for an encoder given its module, class and constructor params
      encoding.encoder = getInstantiatedClass(encoding.encoderModule, encoding.encoderClass, encoding.encoderParams)

      # Take the first part of encoder field name as encoder name
      # Ex: timestamp_weekend.weekend => timestamp_weekend
      encoding.encoder.name = encoding.encoderFieldName.split('.')[0]

      # Add sub-encoder to multi-encoder list
      self.encoder.addEncoder(encoding.dataSourceFieldName, encoding.encoder)

    # If encoder size is not the same to sensor size then throws exception
    encoderSize = self.encoder.getWidth()
    sensorSize = self.width * self.height
    if encoderSize > sensorSize:
      QtGui.QMessageBox.warning(None, "Warning", "'" + self.name + "': Encoder size (" + str(encoderSize) + ") is different from sensor size (" + str(self.width) + " x " + str(self.height) + " = " + str(sensorSize) + ").", QtGui.QMessageBox.Ok)
      return

    return True

  def nextStep(self):
    """
    Performs actions related to time step progression.
    """

    # Update states machine by remove the first element and add a new element in the end
    for encoding in self.encodings:
      encoding.currentValue.rotate()
      if encoding.enableInference:
        encoding.predictedValues.rotate()
        encoding.bestPredictedValue.rotate()

    Node.nextStep(self)
    for bit in self.bits:
      bit.nextStep()

    # Get record value from data source
    # If the last record was reached just rewind it
    data = self.dataSource.getNextRecordDict()
    if not data:
      self.dataSource.rewind()
      data = self.dataSource.getNextRecordDict()

    # Pass raw values to encoder and get a concatenated array
    outputArray = numpy.zeros(self.encoder.getWidth())
    self.encoder.encodeIntoArray(data, outputArray)

    # Get values obtained from the data source.
    outputValues = self.encoder.getScalars(data)

    # Get raw values and respective encoded bit array for each field
    prevOffset = 0
    for i in range(len(self.encodings)):
      encoding = self.encodings[i]

      # Convert the value to its respective data type
      currValue = outputValues[i]
      if encoding.encoderFieldDataType == FieldDataType.boolean:
        currValue = bool(currValue)
      elif encoding.encoderFieldDataType == FieldDataType.integer:
        currValue = int(currValue)
      elif encoding.encoderFieldDataType == FieldDataType.decimal:
        currValue = float(currValue)
      elif encoding.encoderFieldDataType == FieldDataType.dateTime:
        currValue = dateutil.parser.parse(str(currValue))
      elif encoding.encoderFieldDataType == FieldDataType.string:
        currValue = str(currValue)
      encoding.currentValue.setForCurrStep(currValue)

    # Update sensor bits
    for i in range(len(outputArray)):
      if outputArray[i] > 0.:
        self.bits[i].isActive.setForCurrStep(True)
      else:
        self.bits[i].isActive.setForCurrStep(False)

    # Mark falsely predicted bits
    for bit in self.bits:
      if bit.isPredicted.atPreviousStep() and not bit.isActive.atCurrStep():
        bit.isFalselyPredicted.setForCurrStep(True)

    self._output = outputArray

  def getPredictions(self):
    """
    Get the predictions after an iteration.
    """

    if self.predictionsMethod == PredictionsMethod.reconstruction:

      # Prepare list with predictions to be classified
      # This list contains the indexes of all bits that are predicted
      output = []
      for i in range(len(self.bits)):
        if self.bits[i].isPredicted.atCurrStep():
          output.append(1)
        else:
          output.append(0)
      output = numpy.array(output)

      # Decode output and create predictions list
      fieldsDict, fieldsOrder = self.encoder.decode(output)
      for encoding in self.encodings:
        if encoding.enableInference:
          predictions = []
          encoding.predictedValues.setForCurrStep(dict())

          # If encoder field name was returned by decode(), assign the the predictions to it
          if encoding.encoderFieldName in fieldsOrder:
            predictedLabels = fieldsDict[encoding.encoderFieldName][1].split(', ')
            predictedValues = fieldsDict[encoding.encoderFieldName][0]
            for i in range(len(predictedLabels)):
              predictions.append([predictedValues[i], predictedLabels[i]])

          encoding.predictedValues.atCurrStep()[1] = predictions

          # Get the predicted value with the biggest probability to happen
          if len(predictions) > 0:
            bestPredictionRange = predictions[0][0]
            min = bestPredictionRange[0]
            max = bestPredictionRange[1]
            bestPredictedValue = (min + max) / 2.0
            encoding.bestPredictedValue.setForCurrStep(bestPredictedValue)

    elif self.predictionsMethod == PredictionsMethod.classification:
      # A classification involves estimate which are the likely values to occurs in the next time step.

      offset = 0
      for encoding in self.encodings:
        encoderWidth = encoding.encoder.getWidth()

        if encoding.enableInference:
          # Prepare list with predictions to be classified
          # This list contains the indexes of all bits that are predicted
          patternNZ = []
          for i in range(offset, encoderWidth):
            if self.bits[i].isActive.atCurrStep():
              patternNZ.append(i)

          # Get the bucket index of the current value at the encoder
          actualValue = encoding.currentValue.atCurrStep()
          bucketIdx = encoding.encoder.getBucketIndices(actualValue)[0]

          # Perform classification
          clasResults = encoding.classifier.compute(recordNum=Global.currStep, patternNZ=patternNZ, classification={'bucketIdx': bucketIdx, 'actValue': actualValue}, learn=self.enableClassificationLearning, infer=self.enableClassificationInference)

          encoding.predictedValues.setForCurrStep(dict())
          for step in encoding.steps:

            # Calculate probability for each predicted value
            predictions = dict()
            for (actValue, prob) in zip(clasResults['actualValues'], clasResults[step]):
              if actValue in predictions:
                predictions[actValue] += prob
              else:
                predictions[actValue] = prob

            # Remove predictions with low probabilities
            maxVal = (None, None)
            for (actValue, prob) in predictions.items():
              if len(predictions) <= 1:
                break
              if maxVal[0] is None or prob >= maxVal[1]:
                if maxVal[0] is not None and maxVal[1] < encoding.minProbabilityThreshold:
                  del predictions[maxVal[0]]
                maxVal = (actValue, prob)
              elif prob < encoding.minProbabilityThreshold:
                del predictions[actValue]

            # Sort the list of values from more probable to less probable values
            # an decrease the list length to max predictions per step limit
            predictions = sorted(predictions.iteritems(), key=operator.itemgetter(1), reverse=True)
            predictions = predictions[:maxFutureSteps]

            encoding.predictedValues.atCurrStep()[step] = predictions

          # Get the predicted value with the biggest probability to happen
          bestPredictedValue = encoding.predictedValues.atCurrStep()[1][0][0]
          encoding.bestPredictedValue.setForCurrStep(bestPredictedValue)

        offset += encoderWidth

  def calculateStatistics(self):
    """
    Calculate statistics after an iteration.
    """

    if Global.currStep > 0:
      precision = 0.

      # Calculate the prediction precision comparing if the current value is in the range of any prediction.
      for encoding in self.encodings:
        if encoding.enableInference:
          predictions = encoding.predictedValues.atPreviousStep()[1]
          for predictedValue in predictions:
            min = None
            max = None
            value = predictedValue[0]
            if self.predictionsMethod == PredictionsMethod.reconstruction:
              min = value[0]
              max = value[1]
            elif self.predictionsMethod == PredictionsMethod.classification:
              min = value
              max = value
            if isinstance(min, (int, long, float, complex)) and isinstance(max, (int, long, float, complex)):
              min = math.floor(min)
              max = math.ceil(max)
            if min <= encoding.currentValue.atCurrStep() <= max:
              precision = 100.
              break

      # The precision rate is the average of the precision calculated in every step
      self.statsPrecisionRate = (self.statsPrecisionRate + precision) / 2
    else:
      self.statsPrecisionRate = 0.

    for bit in self.bits:
      bit.calculateStatistics()

  #endregion



import collections
import json
from PyQt4 import QtGui, QtCore
from nupic_studio import ArrayTableModel
from nupic_studio.ui import Global
from nupic_studio.htm.encoding import Encoding, FieldDataType

class EncodingForm(QtGui.QDialog):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QDialog.__init__(self)

    self.initUI()

    #region Instance fields

    self.encodingIdx = -1
    """Index of the encoding that is being edited. If index is -1 the user is creating a new encoding."""

    self.encodings = []
    """Temporary list of encodings that is being edited"""

    #endregion

  #endregion

  #region Methods

  def initUI(self):

    # labelDataSourceFieldName
    self.labelDataSourceFieldName = QtGui.QLabel()
    self.labelDataSourceFieldName.setText("Datasource Field Name:")
    self.labelDataSourceFieldName.setAlignment(QtCore.Qt.AlignRight)

    # textBoxDataSourceFieldName
    self.textBoxDataSourceFieldName = QtGui.QLineEdit()
    self.textBoxDataSourceFieldName.setAlignment(QtCore.Qt.AlignLeft)

    # labelDataSourceFieldDataType
    self.labelDataSourceFieldDataType = QtGui.QLabel()
    self.labelDataSourceFieldDataType.setText("Field Data Type:")
    self.labelDataSourceFieldDataType.setAlignment(QtCore.Qt.AlignRight)

    # comboBoxDataSourceFieldDataType
    self.comboBoxDataSourceFieldDataType = QtGui.QComboBox()
    self.comboBoxDataSourceFieldDataType.addItem(FieldDataType.boolean)
    self.comboBoxDataSourceFieldDataType.addItem(FieldDataType.integer)
    self.comboBoxDataSourceFieldDataType.addItem(FieldDataType.decimal)
    self.comboBoxDataSourceFieldDataType.addItem(FieldDataType.dateTime)
    self.comboBoxDataSourceFieldDataType.addItem(FieldDataType.string)

    # checkBoxEnableInference
    self.checkBoxEnableInference = QtGui.QCheckBox()
    self.checkBoxEnableInference.setText("Enable Inference")

    # labelEncoderModule
    self.labelEncoderModule = QtGui.QLabel()
    self.labelEncoderModule.setText("Module:")
    self.labelEncoderModule.setAlignment(QtCore.Qt.AlignRight)

    # textBoxEncoderModule
    self.textBoxEncoderModule = QtGui.QLineEdit()
    self.textBoxEncoderModule.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp('[a-zA-Z0-9._]+')))

    # labelEncoderClass
    self.labelEncoderClass = QtGui.QLabel()
    self.labelEncoderClass.setText("Class:")
    self.labelEncoderClass.setAlignment(QtCore.Qt.AlignRight)

    # textBoxEncoderClass
    self.textBoxEncoderClass = QtGui.QLineEdit()
    self.textBoxEncoderClass.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp('[a-zA-Z0-9_]+')))

    # labelEncoderParams
    self.labelEncoderParams = QtGui.QLabel()
    self.labelEncoderParams.setText("Params:")
    self.labelEncoderParams.setAlignment(QtCore.Qt.AlignRight)

    # dataGridEncoderParams
    data = []
    data.append(['', ''])
    data.append(['', ''])
    data.append(['', ''])
    data.append(['', ''])
    data.append(['', ''])
    data.append(['', ''])
    self.dataGridEncoderParams = QtGui.QTableView()
    self.dataGridEncoderParams.setModel(ArrayTableModel(QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridEncoderParams.setSelectionMode(QtGui.QAbstractItemView.NoSelection)
    self.dataGridEncoderParams.verticalHeader().setDefaultSectionSize(18)
    self.dataGridEncoderParams.model().update(['Parameter', 'Value'], data)
    self.dataGridEncoderParams.resizeColumnsToContents()
    self.dataGridEncoderParams.setMinimumHeight(140)

    # labelEncoderFieldName
    self.labelEncoderFieldName = QtGui.QLabel()
    self.labelEncoderFieldName.setText("Field Name:")
    self.labelEncoderFieldName.setAlignment(QtCore.Qt.AlignRight)

    # textBoxEncoderFieldName
    self.textBoxEncoderFieldName = QtGui.QLineEdit()
    self.textBoxEncoderFieldName.setAlignment(QtCore.Qt.AlignLeft)

    # labelEncoderFieldDataType
    self.labelEncoderFieldDataType = QtGui.QLabel()
    self.labelEncoderFieldDataType.setText("Field Data Type:")
    self.labelEncoderFieldDataType.setAlignment(QtCore.Qt.AlignRight)

    # comboBoxEncoderFieldDataType
    self.comboBoxEncoderFieldDataType = QtGui.QComboBox()
    self.comboBoxEncoderFieldDataType.addItem(FieldDataType.boolean)
    self.comboBoxEncoderFieldDataType.addItem(FieldDataType.integer)
    self.comboBoxEncoderFieldDataType.addItem(FieldDataType.decimal)
    self.comboBoxEncoderFieldDataType.addItem(FieldDataType.dateTime)
    self.comboBoxEncoderFieldDataType.addItem(FieldDataType.string)

    # groupBoxEncoderLayout
    groupBoxEncoderLayout = QtGui.QGridLayout()
    groupBoxEncoderLayout.addWidget(self.labelEncoderModule, 0, 0)
    groupBoxEncoderLayout.addWidget(self.textBoxEncoderModule, 0, 1)
    groupBoxEncoderLayout.addWidget(self.labelEncoderClass, 1, 0)
    groupBoxEncoderLayout.addWidget(self.textBoxEncoderClass, 1, 1)
    groupBoxEncoderLayout.addWidget(self.labelEncoderParams, 2, 0)
    groupBoxEncoderLayout.addWidget(self.dataGridEncoderParams, 2, 1)
    groupBoxEncoderLayout.addWidget(self.labelEncoderFieldName, 3, 0)
    groupBoxEncoderLayout.addWidget(self.textBoxEncoderFieldName, 3, 1)
    groupBoxEncoderLayout.addWidget(self.labelEncoderFieldDataType, 4, 0)
    groupBoxEncoderLayout.addWidget(self.comboBoxEncoderFieldDataType, 4, 1)

    # groupBoxEncoder
    self.groupBoxEncoder = QtGui.QGroupBox()
    self.groupBoxEncoder.setLayout(groupBoxEncoderLayout)
    self.groupBoxEncoder.setTitle("Encoder")

    # groupBoxMainLayout
    groupBoxMainLayout = QtGui.QGridLayout()
    groupBoxMainLayout.addWidget(self.labelDataSourceFieldName, 0, 0)
    groupBoxMainLayout.addWidget(self.textBoxDataSourceFieldName, 0, 1)
    groupBoxMainLayout.addWidget(self.labelDataSourceFieldDataType, 1, 0)
    groupBoxMainLayout.addWidget(self.comboBoxDataSourceFieldDataType, 1, 1)
    groupBoxMainLayout.addWidget(self.checkBoxEnableInference, 2, 1)
    groupBoxMainLayout.addWidget(self.groupBoxEncoder, 3, 1)

    # groupBoxMain
    self.groupBoxMain = QtGui.QGroupBox()
    self.groupBoxMain.setLayout(groupBoxMainLayout)

    # buttonBox
    self.buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).clicked.connect(self.__buttonOk_Click)
    self.buttonBox.button(QtGui.QDialogButtonBox.Ok).setEnabled(not Global.simulationInitialized)
    self.buttonBox.button(QtGui.QDialogButtonBox.Cancel).clicked.connect(self.__buttonCancel_Click)

    # layout
    layout = QtGui.QVBoxLayout()
    layout.addWidget(self.groupBoxMain)
    layout.addWidget(self.buttonBox)

    # SensorForm
    self.setLayout(layout)
    self.setModal(True)
    self.setWindowTitle("Sensor Properties")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.resize(400, 200)

  def setControlsValues(self):
    """
    Set controls values from a class instance.
    """

    # Set controls value with encoding params
    if self.encodingIdx >= 0:
      encoding = self.encodings[self.encodingIdx]
      self.checkBoxEnableInference.setChecked(encoding.enableInference)
      self.textBoxDataSourceFieldName.setText(encoding.dataSourceFieldName)
      self.comboBoxDataSourceFieldDataType.setCurrentIndex(self.comboBoxDataSourceFieldDataType.findText(encoding.dataSourceFieldDataType, QtCore.Qt.MatchFixedString))

      # Set encoding parameters
      self.textBoxEncoderModule.setText(encoding.encoderModule)
      self.textBoxEncoderClass.setText(encoding.encoderClass)
      encoderParams = json.loads(encoding.encoderParams.replace("'", "\""), object_pairs_hook=collections.OrderedDict)
      gridData = self.dataGridEncoderParams.model().data
      row = 0
      for key, value in encoderParams.iteritems():
        gridData[row][0] = key
        gridData[row][1] = value
        row += 1
      self.textBoxEncoderFieldName.setText(encoding.encoderFieldName)
      self.comboBoxEncoderFieldDataType.setCurrentIndex(self.comboBoxEncoderFieldDataType.findText(encoding.encoderFieldDataType, QtCore.Qt.MatchFixedString))

  def duplicatedFieldName(self, fieldName):
    """
    Check if exists an encoding with the same name.
    """
    duplicated = False

    if len(self.encodings) > 0:
      for i in range(len(self.encodings)):
        if self.encodings[i].encoderFieldName == fieldName and i != self.encodingIdx:
          duplicated = True
          break

    return duplicated

  #endregion

  #region Events

  def __buttonOk_Click(self, event):
    """
    Check if values changed and save the,.
    """

    encoderParamsDict = collections.OrderedDict()
    if self.textBoxDataSourceFieldName.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Record field name was not specified.")
      return
    elif self.textBoxEncoderModule.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Encoder module was not specified.")
      return
    elif self.textBoxEncoderClass.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Encoder class was not specified.")
      return
    elif self.textBoxEncoderFieldName.text() == '':
      QtGui.QMessageBox.warning(self, "Warning", "Encoder field name was not specified.")
      return
    elif self.duplicatedFieldName(self.textBoxEncoderFieldName.text()):
      QtGui.QMessageBox.warning(self, "Warning", "Encoder field name already is used by other encoding.")
      return
    else:
      gridData = self.dataGridEncoderParams.model().data
      for row in range(len(gridData)):
        if gridData[row][0] != '':
          # Valid parameter name
          try:
            gridData[row][0] = gridData[row][0].toString()
          except:
            pass
          param = str(gridData[row][0])
          validExpr = QtCore.QRegExp('[a-zA-Z0-9_]+')
          if not validExpr.exactMatch(param):
            QtGui.QMessageBox.warning(self, "Warning", "'" + param + "' is not a valid name.")
            return

          # Valid parameter value
          try:
            gridData[row][1] = gridData[row][1].toString()
          except:
            pass
          value = str(gridData[row][1])
          if len(value) == 0:
            QtGui.QMessageBox.warning(self, "Warning", "'" + param + "' value is empty.")
            return

          # Add param name and its value to dictionary
          encoderParamsDict[param] = value

    dataSourceFieldName = str(self.textBoxDataSourceFieldName.text())
    dataSourceFieldDataType = str(self.comboBoxDataSourceFieldDataType.currentText())
    enableInference = self.checkBoxEnableInference.isChecked()
    encoderModule = str(self.textBoxEncoderModule.text())
    encoderClass = str(self.textBoxEncoderClass.text())
    encoderParams = json.dumps(encoderParamsDict)
    encoderFieldName = str(self.textBoxEncoderFieldName.text())
    encoderFieldDataType = str(self.comboBoxEncoderFieldDataType.currentText())

    # Remove double quotes from param values
    encoderParams = encoderParams.replace("\"", "'")
    encoderParams = encoderParams.replace("True", "true")
    encoderParams = encoderParams.replace("False", "false")

    # If this is a new encoding get it from list else create a new one
    if self.encodingIdx >= 0:
      encoding = self.encodings[self.encodingIdx]
    else:
      encoding = Encoding()
      self.encodings.append(encoding)

    # If anything has changed
    if encoding.dataSourceFieldName != dataSourceFieldName or encoding.dataSourceFieldDataType != dataSourceFieldDataType or encoding.enableInference != enableInference or encoding.encoderModule != encoderModule or encoding.encoderClass != encoderClass or encoding.encoderParams != encoderParams or encoding.encoderFieldName != encoderFieldName or encoding.encoderFieldDataType != encoderFieldDataType:
      # Set encoding params with controls values
      encoding.dataSourceFieldName = dataSourceFieldName
      encoding.dataSourceFieldDataType = dataSourceFieldDataType
      encoding.enableInference = enableInference
      encoding.encoderModule = encoderModule
      encoding.encoderClass = encoderClass
      encoding.encoderParams = encoderParams
      encoding.encoderFieldName = encoderFieldName
      encoding.encoderFieldDataType = encoderFieldDataType

      self.accept()

    self.close()

  def __buttonCancel_Click(self, event):
    self.reject()
    self.close()

  #endregion



import pyqtgraph as pg
from PyQt4 import Qt, QtGui, QtCore
from nupic_studio import ArrayTableModel
from nupic_studio.ui import Global
from nupic_studio.htm import maxPreviousSteps, maxFutureSteps, maxPreviousStepsWithInference
from nupic_studio.htm.node import NodeType, Node
from nupic_studio.htm.node_sensor import PredictionsMethod
from nupic_studio.htm.encoding import FieldDataType

class NodeInformationForm(QtGui.QWidget):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QWidget.__init__(self)

    #region Instance fields

    self.previousSelectedNode = None
    self.selectedSensor = None
    self.selectedEncoding = None
    self.selectedRegion = None
    self.selectedColumn = None
    self.selectedProximalSynapse = None
    self.selectedCell = None
    self.selectedDistalSegment = None
    self.selectedDistalSynapse = None

    # Predictions variables used for the predictions chart
    self.currentValuesPlotItem = None
    self.predictedValuesPlotItem = None

    #endregion

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # labelSensorName
    self.labelSensorName = QtGui.QLabel()
    self.labelSensorName.setText("Name")
    self.labelSensorName.setAlignment(QtCore.Qt.AlignRight)

    # textBoxSensorName
    self.textBoxSensorName = QtGui.QLineEdit()
    self.textBoxSensorName.setEnabled(False)
    self.textBoxSensorName.setAlignment(QtCore.Qt.AlignLeft)

    # labelSensorPrecisionRate
    self.labelSensorPrecisionRate = QtGui.QLabel()
    self.labelSensorPrecisionRate.setText("Precision Rate (%)")
    self.labelSensorPrecisionRate.setAlignment(QtCore.Qt.AlignRight)

    # textBoxSensorPrecisionRate
    self.textBoxSensorPrecisionRate = QtGui.QLineEdit()
    self.textBoxSensorPrecisionRate.setEnabled(False)
    self.textBoxSensorPrecisionRate.setAlignment(QtCore.Qt.AlignRight)

    # checkBoxEnableClassificationLearning
    self.checkBoxEnableClassificationLearning = QtGui.QCheckBox()
    self.checkBoxEnableClassificationLearning.setText("Enable Classification Learning")
    self.checkBoxEnableClassificationLearning.toggled.connect(self.__checkBoxEnableClassificationLearning_Toggled)

    # checkBoxEnableClassificationInference
    self.checkBoxEnableClassificationInference = QtGui.QCheckBox()
    self.checkBoxEnableClassificationInference.setText("Enable Classification Inference")
    self.checkBoxEnableClassificationInference.toggled.connect(self.__checkBoxEnableClassificationInference_Toggled)

    # tabPageSensor1Layout
    tabPageSensor1Layout = QtGui.QGridLayout()
    tabPageSensor1Layout.addWidget(self.labelSensorName, 0, 0)
    tabPageSensor1Layout.addWidget(self.textBoxSensorName, 0, 1)
    tabPageSensor1Layout.addWidget(self.labelSensorPrecisionRate, 1, 0)
    tabPageSensor1Layout.addWidget(self.textBoxSensorPrecisionRate, 1, 1)
    tabPageSensor1Layout.addWidget(self.checkBoxEnableClassificationLearning, 2, 1)
    tabPageSensor1Layout.addWidget(self.checkBoxEnableClassificationInference, 3, 1)
    tabPageSensor1Layout.setRowStretch(4, 100)

    # tabPageSensorLayout
    tabPageSensorLayout = QtGui.QGridLayout()
    tabPageSensorLayout.addLayout(tabPageSensor1Layout, 0, 0)
    tabPageSensorLayout.setColumnStretch(1, 100)

    # tabPageSensor
    self.tabPageSensor = QtGui.QWidget()
    self.tabPageSensor.setLayout(tabPageSensorLayout)

    # dataGridBits
    self.dataGridBits = QtGui.QTableView()
    self.dataGridBits.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridBits.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridBits.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridBits.setToolTip("Click on a row to see more details")
    self.dataGridBits.verticalHeader().setDefaultSectionSize(20)

    # tabPageBitsLayout
    tabPageBitsLayout = QtGui.QHBoxLayout()
    tabPageBitsLayout.addWidget(self.dataGridBits)

    # tabPageBits
    self.tabPageBits = QtGui.QWidget()
    self.tabPageBits.setLayout(tabPageBitsLayout)

    # labelEncoding
    self.labelEncoding = QtGui.QLabel()
    self.labelEncoding.setText("Encoding:")
    self.labelEncoding.setAlignment(QtCore.Qt.AlignRight)

    # comboBoxEncoding
    self.comboBoxEncoding = QtGui.QComboBox()
    self.comboBoxEncoding.currentIndexChanged.connect(self.__comboBoxEncoding_CurrentIndexChanged)

    # labelCurrentValue
    self.labelCurrentValue = QtGui.QLabel()
    self.labelCurrentValue.setText("Current Value")
    self.labelCurrentValue.setAlignment(QtCore.Qt.AlignRight)

    # textBoxCurrentValue
    self.textBoxCurrentValue = QtGui.QLineEdit()
    self.textBoxCurrentValue.setEnabled(False)
    self.textBoxCurrentValue.setAlignment(QtCore.Qt.AlignRight)

    # labelPredictedValues
    self.labelPredictedValues = QtGui.QLabel()
    self.labelPredictedValues.setText("Predicted Values")
    self.labelPredictedValues.setAlignment(QtCore.Qt.AlignRight)

    # dataGridPredictedValues
    self.dataGridPredictedValues = QtGui.QTableView()
    self.dataGridPredictedValues.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridPredictedValues.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridPredictedValues.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridPredictedValues.verticalHeader().setDefaultSectionSize(20)

    # encoding1Layout
    encoding1Layout = QtGui.QGridLayout()
    encoding1Layout.addWidget(self.labelEncoding, 0, 0)
    encoding1Layout.addWidget(self.comboBoxEncoding, 0, 1)
    encoding1Layout.addWidget(self.labelCurrentValue, 1, 0)
    encoding1Layout.addWidget(self.textBoxCurrentValue, 1, 1)
    encoding1Layout.setRowStretch(2, 100)

    # sliderStep
    self.sliderStep = QtGui.QSlider()
    self.sliderStep.setOrientation(QtCore.Qt.Horizontal)
    self.sliderStep.setSingleStep(1)
    self.sliderStep.setRange(1, maxFutureSteps)
    self.sliderStep.valueChanged.connect(self.__sliderStep_ValueChanged)

    # encoding2Layout
    encoding2Layout = QtGui.QGridLayout()
    encoding2Layout.addWidget(self.sliderStep, 0, 1)
    encoding2Layout.addWidget(self.labelPredictedValues, 1, 0)
    encoding2Layout.addWidget(self.dataGridPredictedValues, 1, 1)

    # predictionsChart
    self.predictionsChart = pg.PlotWidget()
    self.predictionsChart.showGrid(x=True, y=True)
    self.predictionsChart.getAxis('left').setGrid(1)

    # encoding3Layout
    encoding3Layout = QtGui.QGridLayout()
    encoding3Layout.addWidget(self.predictionsChart, 0, 1)

    # tabPageEncodingsLayout
    tabPageEncodingsLayout = QtGui.QHBoxLayout()
    tabPageEncodingsLayout.addLayout(encoding1Layout)
    tabPageEncodingsLayout.addLayout(encoding2Layout)
    tabPageEncodingsLayout.addLayout(encoding3Layout)

    # tabPageEncodings
    self.tabPageEncodings = QtGui.QWidget()
    self.tabPageEncodings.setLayout(tabPageEncodingsLayout)

    # labelRegionName
    self.labelRegionName = QtGui.QLabel()
    self.labelRegionName.setText("Name")
    self.labelRegionName.setAlignment(QtCore.Qt.AlignRight)

    # textBoxRegionName
    self.textBoxRegionName = QtGui.QLineEdit()
    self.textBoxRegionName.setEnabled(False)
    self.textBoxRegionName.setAlignment(QtCore.Qt.AlignLeft)

    # labelRegionPrecisionRate
    self.labelRegionPrecisionRate = QtGui.QLabel()
    self.labelRegionPrecisionRate.setText("Precision Rate (%)")
    self.labelRegionPrecisionRate.setAlignment(QtCore.Qt.AlignRight)

    # textBoxRegionPrecisionRate
    self.textBoxRegionPrecisionRate = QtGui.QLineEdit()
    self.textBoxRegionPrecisionRate.setEnabled(False)
    self.textBoxRegionPrecisionRate.setAlignment(QtCore.Qt.AlignRight)

    # checkBoxEnableSpatialLearning
    self.checkBoxEnableSpatialLearning = QtGui.QCheckBox()
    self.checkBoxEnableSpatialLearning.setText("Enable Spatial Learning")
    self.checkBoxEnableSpatialLearning.toggled.connect(self.__checkBoxEnableSpatialLearning_Toggled)

    # checkBoxEnableTemporalLearning
    self.checkBoxEnableTemporalLearning = QtGui.QCheckBox()
    self.checkBoxEnableTemporalLearning.setText("Enable Temporal Learning")
    self.checkBoxEnableTemporalLearning.toggled.connect(self.__checkBoxEnableTemporalLearning_Toggled)

    # tabPageRegionsLayout
    tabPageRegionsLayout = QtGui.QGridLayout()
    tabPageRegionsLayout.addWidget(self.labelRegionName, 0, 0)
    tabPageRegionsLayout.addWidget(self.textBoxRegionName, 0, 1)
    tabPageRegionsLayout.addWidget(self.labelRegionPrecisionRate, 1, 0)
    tabPageRegionsLayout.addWidget(self.textBoxRegionPrecisionRate, 1, 1)
    tabPageRegionsLayout.addWidget(self.checkBoxEnableSpatialLearning, 2, 1)
    tabPageRegionsLayout.addWidget(self.checkBoxEnableTemporalLearning, 3, 1)
    tabPageRegionsLayout.setRowStretch(4, 100)
    tabPageRegionsLayout.setColumnStretch(2, 100)

    # tabPageRegions
    self.tabPageRegions = QtGui.QWidget()
    self.tabPageRegions.setLayout(tabPageRegionsLayout)

    # dataGridColumns
    self.dataGridColumns = QtGui.QTableView()
    self.dataGridColumns.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridColumns.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridColumns.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridColumns.setToolTip("Click on a row to see more details")
    self.dataGridColumns.verticalHeader().setDefaultSectionSize(20)
    self.dataGridColumns.selectionModel().selectionChanged.connect(self.__dataGridColumns_SelectionChanged)

    # tabPageColumnsLayout
    tabPageColumnsLayout = QtGui.QHBoxLayout()
    tabPageColumnsLayout.addWidget(self.dataGridColumns)

    # tabPageColumns
    self.tabPageColumns = QtGui.QWidget()
    self.tabPageColumns.setLayout(tabPageColumnsLayout)

    # dataGridProximalSynapses
    self.dataGridProximalSynapses = QtGui.QTableView()
    self.dataGridProximalSynapses.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridProximalSynapses.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridProximalSynapses.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridProximalSynapses.setToolTip("Click on a row to see more details")
    self.dataGridProximalSynapses.verticalHeader().setDefaultSectionSize(20)
    self.dataGridProximalSynapses.selectionModel().selectionChanged.connect(self.__dataGridProximalSynapses_SelectionChanged)

    # tabPageProximalSynapsesLayout
    tabPageProximalSynapsesLayout = QtGui.QHBoxLayout()
    tabPageProximalSynapsesLayout.addWidget(self.dataGridProximalSynapses)

    # tabPageProximalSynapses
    self.tabPageProximalSynapses = QtGui.QWidget()
    self.tabPageProximalSynapses.setLayout(tabPageProximalSynapsesLayout)

    # dataGridCells
    self.dataGridCells = QtGui.QTableView()
    self.dataGridCells.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridCells.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridCells.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridCells.setToolTip("Click on a row to see more details")
    self.dataGridCells.verticalHeader().setDefaultSectionSize(20)
    self.dataGridCells.selectionModel().selectionChanged.connect(self.__dataGridCells_SelectionChanged)

    # tabPageCellsLayout
    tabPageCellsLayout = QtGui.QHBoxLayout()
    tabPageCellsLayout.addWidget(self.dataGridCells)

    # tabPageCells
    self.tabPageCells = QtGui.QWidget()
    self.tabPageCells.setLayout(tabPageCellsLayout)

    # dataGridDistalSegments
    self.dataGridDistalSegments = QtGui.QTableView()
    self.dataGridDistalSegments.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridDistalSegments.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridDistalSegments.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridDistalSegments.setToolTip("Click on a row to see more details")
    self.dataGridDistalSegments.verticalHeader().setDefaultSectionSize(20)
    self.dataGridDistalSegments.selectionModel().selectionChanged.connect(self.__dataGridDistalSegments_SelectionChanged)

    # tabPageDistalSegmentsLayout
    tabPageDistalSegmentsLayout = QtGui.QHBoxLayout()
    tabPageDistalSegmentsLayout.addWidget(self.dataGridDistalSegments)

    # tabPageDistalSegments
    self.tabPageDistalSegments = QtGui.QWidget()
    self.tabPageDistalSegments.setLayout(tabPageDistalSegmentsLayout)

    # dataGridDistalSynapses
    self.dataGridDistalSynapses = QtGui.QTableView()
    self.dataGridDistalSynapses.setModel(ArrayTableModel(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable))
    self.dataGridDistalSynapses.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)
    self.dataGridDistalSynapses.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
    self.dataGridDistalSynapses.setToolTip("Click on a row to see more details")
    self.dataGridDistalSynapses.verticalHeader().setDefaultSectionSize(20)
    self.dataGridDistalSynapses.selectionModel().selectionChanged.connect(self.__dataGridDistalSynapses_SelectionChanged)

    # tabPageDistalSynapsesLayout
    tabPageDistalSynapsesLayout = QtGui.QHBoxLayout()
    tabPageDistalSynapsesLayout.addWidget(self.dataGridDistalSynapses)

    # tabPageDistalSynapses
    self.tabPageDistalSynapses = QtGui.QWidget()
    self.tabPageDistalSynapses.setLayout(tabPageDistalSynapsesLayout)

    # tabControlMain
    self.tabControlMain = QtGui.QTabWidget()

    # layout
    layout = QtGui.QHBoxLayout()
    layout.addWidget(self.tabControlMain)

    # NodeInformationForm
    self.setLayout(layout)
    self.setWindowTitle("Node Information")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.setMinimumHeight(200)
    self.setMaximumHeight(300)

  def clearControls(self):
    """
    Reset all controls.
    """

    self.textBoxCurrentValue.setText("")
    self.sliderStep.setEnabled(False)
    self.dataGridPredictedValues.model().update([], [])
    self.predictionsChart.setVisible(False)

  def refreshControls(self):
    """
    Refresh controls for each time step.
    """

    selectedNode = Global.architectureForm.designPanel.selectedNode

    # Show information according to note type
    if selectedNode != self.previousSelectedNode:
      while True:
        self.tabControlMain.removeTab(0)
        if self.tabControlMain.count() == 0:
          break
      if selectedNode.type == NodeType.region:
        self.selectedRegion = selectedNode

        self.textBoxRegionName.setText(self.selectedRegion.name)
        self.checkBoxEnableSpatialLearning.setChecked(self.selectedRegion.enableSpatialLearning)
        self.checkBoxEnableTemporalLearning.setChecked(self.selectedRegion.enableTemporalLearning)
        self.showTab(self.tabPageRegions, "Region")
        self.showTab(self.tabPageColumns, "Columns")
        self.dataGridColumns.clearSelection()
      elif selectedNode.type == NodeType.sensor:
        self.selectedSensor = selectedNode

        self.textBoxSensorName.setText(self.selectedSensor.name)
        self.showTab(self.tabPageSensor, "Sensor")
        self.showTab(self.tabPageBits, "Bits")
        self.showTab(self.tabPageEncodings, "Encodings")
        self.dataGridBits.clearSelection()
        if self.selectedSensor.predictionsMethod == PredictionsMethod.classification:
          self.checkBoxEnableClassificationLearning.setVisible(True)
          self.checkBoxEnableClassificationLearning.setChecked(self.selectedSensor.enableClassificationLearning)
          self.checkBoxEnableClassificationInference.setVisible(True)
          self.checkBoxEnableClassificationInference.setChecked(self.selectedSensor.enableClassificationInference)
        else:
          self.checkBoxEnableClassificationLearning.setVisible(False)
          self.checkBoxEnableClassificationInference.setVisible(False)

        # Populate encodings combobox
        self.comboBoxEncoding.clear()
        for encoding in self.selectedSensor.encodings:
          if encoding.enableInference:
            name = encoding.encoderFieldName.split('.')[0]
            self.comboBoxEncoding.addItem(name)
        self.selectedEncoding = None
      self.tabControlMain.selectedIndex = 0

      self.previousSelectedNode = selectedNode

    if Global.simulationInitialized:
      if selectedNode.type == NodeType.region:
        self.textBoxRegionPrecisionRate.setText("{0:.3f}".format(self.selectedRegion.statsPrecisionRate))

        # Bind the columns from this region
        header, data = self.getColumnsData(self.selectedRegion)
        self.dataGridColumns.model().update(header, data)
        self.dataGridColumns.resizeColumnsToContents()

      elif selectedNode.type == NodeType.sensor:
        self.textBoxSensorPrecisionRate.setText("{0:.3f}".format(self.selectedSensor.statsPrecisionRate))

        # Bind the bits from this sensor
        header, data = self.getBitsData(self.selectedSensor)
        self.dataGridBits.model().update(header, data)
        self.dataGridBits.resizeColumnsToContents()

        # Reset step slider state
        self.sliderStep.setEnabled(True)
        self.sliderStep.setValue(self.sliderStep.minimum())
        if self.selectedSensor.predictionsMethod == PredictionsMethod.classification:
          self.sliderStep.setEnabled(True)
        else:
          self.sliderStep.setEnabled(False)

        # Set default encoding
        if self.selectedEncoding == None:
          self.selectedEncoding = self.selectedSensor.encodings[0]
        self.updateEncodingControls()

  def formatValue(self, dataType, value):
    formattedValue = None

    if value == None:
      formattedValue = "None"
    elif dataType == FieldDataType.boolean:
      if value == 0:
        formattedValue = "False"
      else:
        formattedValue = "True"
    elif dataType == FieldDataType.integer:
      formattedValue = "{0}".format(value)
    elif dataType == FieldDataType.decimal:
      formattedValue = "{0:.3f}".format(value)
    elif dataType == FieldDataType.dateTime:
      formattedValue = value.strftime("%Y-%m-%d %H:%M:%S")
    else:
      formattedValue = str(value)

    return formattedValue

  def getBitsData(self, selectedSensor):
    header = ['Pos (x,y)', 'Was Predicted', 'Is Active', 'Activation Rate', 'Precision Rate']
    data = []
    for bit in selectedSensor.bits:
      pos = str(bit.x) + ", " + str(bit.y)
      wasPredicted = bit.isPredicted.atGivenStepAgo(Global.selStep + 1)
      isActive = bit.isActive.atGivenStepAgo(Global.selStep)
      activationRate = "{0:.3f}".format(bit.statsActivationRate)
      precisionRate = "{0:.3f}".format(bit.statsPrecisionRate)
      data.append([pos, wasPredicted, isActive, activationRate, precisionRate])

    return header, data

  def updateEncodingControls(self):
    self.textBoxCurrentValue.setText(self.formatValue(self.selectedEncoding.encoderFieldDataType, self.selectedEncoding.currentValue.atGivenStepAgo(Global.selStep)))
    self.sliderStep.setVisible(self.selectedEncoding.enableInference)
    self.labelPredictedValues.setVisible(self.selectedEncoding.enableInference)
    self.dataGridPredictedValues.setVisible(self.selectedEncoding.enableInference)
    self.predictionsChart.setVisible(False)
    if Global.simulationInitialized and self.selectedEncoding.enableInference:
      self.updatePredictedValuesGrid()
      if self.selectedEncoding.encoderFieldDataType == FieldDataType.integer or self.selectedEncoding.encoderFieldDataType == FieldDataType.decimal:
        self.updatePredictionsChart()
        self.predictionsChart.setVisible(True)

  def updatePredictedValuesGrid(self):
    step = self.sliderStep.value()
    header, data = self.getPredictedValuesData(step)
    self.dataGridPredictedValues.model().update(header, data)
    self.dataGridPredictedValues.resizeColumnsToContents()

  def updatePredictionsChart(self):

    # Update the chart with the updated predictions history
    if self.currentValuesPlotItem == None:
      # Set plot lines
      self.currentValuesPlotItem = self.predictionsChart.plot(Global.timeStepsPredictionsChart.getList(), self.selectedEncoding.currentValue.getList())
      self.currentValuesPlotItem.setPen(QtGui.QColor.fromRgb(0, 100, 0)) # green color
      self.predictedValuesPlotItem = self.predictionsChart.plot(Global.timeStepsPredictionsChart.getList(), self.selectedEncoding.bestPredictedValue.getList())
      self.predictedValuesPlotItem.setPen(QtGui.QColor.fromRgb(255, 215, 80)) # yellow color

      # Set legend
      legend = self.predictionsChart.addLegend(size=None, offset=(0, 0))
      legend.addItem(self.currentValuesPlotItem, "Current")
      legend.addItem(self.predictedValuesPlotItem, "Predicted")
    else:
      self.currentValuesPlotItem.setData(Global.timeStepsPredictionsChart.getList(), self.selectedEncoding.currentValue.getList())
      self.predictedValuesPlotItem.setData(Global.timeStepsPredictionsChart.getList(), self.selectedEncoding.bestPredictedValue.getList())

    # Set X axis visible range
    minX = Global.timeStepsPredictionsChart.atFirstStep()
    maxX = minX + maxPreviousStepsWithInference
    maxX += 30 # Increase space to avoid plot lines overlap the legend
    self.predictionsChart.setXRange(minX, maxX)

  def getPredictedValuesData(self, futureStep):
    header = []
    if self.selectedSensor.predictionsMethod == PredictionsMethod.reconstruction:
      header = ['Value']
    elif self.selectedSensor.predictionsMethod == PredictionsMethod.classification:
      header = ['Value', 'Probability']

    data = []
    predictions = self.selectedEncoding.predictedValues.atGivenStepAgo(Global.selStep)[futureStep]
    for predictedValue in predictions:
      if self.selectedSensor.predictionsMethod == PredictionsMethod.reconstruction:
        value = predictedValue[1]
        data.append([value])
      elif self.selectedSensor.predictionsMethod == PredictionsMethod.classification:
        value = self.formatValue(self.selectedEncoding.encoderFieldDataType, predictedValue[0])
        probability = "{0:.3f}".format(predictedValue[1] * 100)
        data.append([value, probability])

    return header, data

  def getColumnsData(self, selectedRegion):
    header = ['Pos (x,y)', 'Was Predicted', 'Is Active', 'Activation Rate', 'Precision Rate']
    data = []
    for column in selectedRegion.columns:
      pos = str(column.x) + ", " + str(column.y)
      wasPredicted = column.segment.isPredicted.atGivenStepAgo(Global.selStep + 1)
      isActive = column.segment.isActive.atGivenStepAgo(Global.selStep)
      activationRate = "{0:.3f}".format(column.segment.statsActivationRate)
      precisionRate = "{0:.3f}".format(column.segment.statsPrecisionRate)
      data.append([pos, wasPredicted, isActive, activationRate, precisionRate])

    return header, data

  def getProximalSynapsesData(self, selectedSegment):
    #TODO: Put sensor bit position (x,y,z)
    header = ['Permanence', 'Is Connected', 'Connection Rate', 'Precision Rate']
    data = []
    for synapse in selectedSegment.synapses:
      permanence = "{0:.3f}".format(synapse.permanence.atGivenStepAgo(Global.selStep))
      isConnected = synapse.isConnected.atGivenStepAgo(Global.selStep)
      connectionRate = "{0:.3f}".format(synapse.statsConnectionRate)
      precisionRate = "{0:.3f}".format(synapse.statsPrecisionRate)
      data.append([permanence, isConnected, connectionRate, precisionRate])

    return header, data

  def getCellsData(self, selectedColumn):
    header = ['Pos (z)', 'Was Predicted', 'Is Active', 'Activation Rate', 'Precision Rate']
    data = []
    for cell in selectedColumn.cells:
      pos = str(cell.z)
      wasPredicted = cell.isPredicted.atGivenStepAgo(Global.selStep + 1)
      isActive = cell.isActive.atGivenStepAgo(Global.selStep)
      activationRate = "{0:.3f}".format(cell.statsActivationRate)
      precisionRate = "{0:.3f}".format(cell.statsPrecisionRate)
      data.append([pos, wasPredicted, isActive, activationRate, precisionRate])

    return header, data

  def getDistalSegmetsData(self, selectedCell):
    header = ['Is Active', 'Activation Rate', 'Activation Rate']
    data = []
    for segment in selectedCell.segments:
      isActive = segment.isActive.atGivenStepAgo(Global.selStep)
      activationRate = "{0:.3f}".format(segment.statsActivationRate)
      data.append([isActive, activationRate, activationRate])

    return header, data

  def getDistalSynapsesData(self, selectedSegment):
    #TODO: Put lateral cell position (x,y,z)
    header = ['Permanence', 'Is Connected', 'Connection Rate']
    data = []
    for synapse in selectedSegment.synapses:
      permanence = "{0:.3f}".format(synapse.permanence.atGivenStepAgo(Global.selStep))
      isConnected = synapse.isConnected.atGivenStepAgo(Global.selStep)
      connectionRate = "{0:.3f}".format(synapse.statsConnectionRate)
      data.append([permanence, isConnected, connectionRate])

    return header, data

  def showTab(self, tab, title):
    tabFound = False
    for tabIdx in range(self.tabControlMain.count()):
      if self.tabControlMain.tabText(tabIdx) == title:
        tabFound = True
    if not tabFound:
      self.tabControlMain.addTab(tab, title)

  #endregion

  #region Events

  def closeEvent(self, event):
    self.Hide()
    self.Parent = None
    event.Cancel = True

  def __sliderStep_ValueChanged(self, value):
    self.updatePredictedValuesGrid()

  def __comboBoxEncoding_CurrentIndexChanged(self, event):
    if Global.simulationInitialized:
      encodingIdx = self.comboBoxEncoding.currentIndex()
      self.selectedEncoding = self.selectedSensor.encodings[encodingIdx]
      self.updateEncodingControls()

  def __checkBoxEnableSpatialLearning_Toggled(self, event):
    self.selectedRegion.enableSpatialLearning = self.checkBoxEnableSpatialLearning.isChecked()

  def __checkBoxEnableTemporalLearning_Toggled(self, event):
    self.selectedRegion.enableTemporalLearning = self.checkBoxEnableTemporalLearning.isChecked()

  def __checkBoxEnableClassificationLearning_Toggled(self, event):
    self.selectedSensor.enableClassificationLearning = self.checkBoxEnableClassificationLearning.isChecked()

  def __checkBoxEnableClassificationInference_Toggled(self, event):
    self.selectedSensor.enableClassificationInference = self.checkBoxEnableClassificationInference.isChecked()

  def __dataGridColumns_SelectionChanged(self, event):
    if self.selectedColumn != None:
      self.selectedColumn.segment.tree3d_selected = False

    self.dataGridProximalSynapses.clearSelection()
    self.dataGridCells.clearSelection()

    selectedRows = self.dataGridColumns.selectionModel().selectedRows()
    if len(selectedRows) > 0:
      index = selectedRows[0].row()
      self.selectedColumn = self.selectedRegion.columns[index]
      self.selectedColumn.segment.tree3d_selected = True

      # Bind the synapses of the selected segment
      self.showTab(self.tabPageProximalSynapses, "Proximal Synapses")
      header, data = self.getProximalSynapsesData(self.selectedColumn.segment)
      self.dataGridProximalSynapses.model().update(header, data)
      self.dataGridProximalSynapses.resizeColumnsToContents()

      # Bind the cells of the selected column
      self.showTab(self.tabPageCells, "Cells")
      header, data = self.getCellsData(self.selectedColumn)
      self.dataGridCells.model().update(header, data)
      self.dataGridCells.resizeColumnsToContents()

    Global.simulationForm.refreshControls()

  def __dataGridProximalSynapses_SelectionChanged(self, event):
    if self.selectedProximalSynapse != None:
      self.selectedProximalSynapse.tree3d_selected = False

    selectedRows = self.dataGridProximalSynapses.selectionModel().selectedRows()
    if len(selectedRows) > 0:
      index = selectedRows[0].row()
      self.selectedProximalSynapse = self.selectedColumn.segment.synapses[index]
      self.selectedProximalSynapse.tree3d_selected = True

    Global.simulationForm.refreshControls()

  def __dataGridCells_SelectionChanged(self, event):
    if self.selectedCell != None:
      self.selectedCell.tree3d_selected = False

    self.dataGridDistalSegments.clearSelection()

    selectedRows = self.dataGridCells.selectionModel().selectedRows()
    if len(selectedRows) > 0:
      index = selectedRows[0].row()
      self.selectedCell = self.selectedColumn.cells[index]
      self.selectedCell.tree3d_selected = True

      # Bind the segments of the selected cell
      self.showTab(self.tabPageDistalSegments, "Distal Segments")
      header, data = self.getDistalSegmetsData(self.selectedCell)
      self.dataGridDistalSegments.model().update(header, data)
      self.dataGridDistalSegments.resizeColumnsToContents()

    Global.simulationForm.refreshControls()

  def __dataGridDistalSegments_SelectionChanged(self, event):
    if self.selectedDistalSegment != None:
      self.selectedDistalSegment.tree3d_selected = False

    self.dataGridDistalSynapses.clearSelection()

    selectedRows = self.dataGridDistalSegments.selectionModel().selectedRows()
    if len(selectedRows) > 0:
      index = selectedRows[0].row()
      self.selectedDistalSegment = self.selectedCell.segments[index]

      # Bind the synapses of the selected segment
      self.showTab(self.tabPageDistalSynapses, "Distal Synapses")
      header, data = self.getDistalSynapsesData(self.selectedDistalSegment)
      self.dataGridDistalSynapses.model().update(header, data)
      self.dataGridDistalSynapses.resizeColumnsToContents()

    Global.simulationForm.refreshControls()

  def __dataGridDistalSynapses_SelectionChanged(self, event):
    if self.selectedDistalSynapse != None:
      self.selectedDistalSynapse.tree3d_selected = False

    selectedRows = self.dataGridDistalSynapses.selectionModel().selectedRows()
    if len(selectedRows) > 0:
      index = selectedRows[0].row()
      self.selectedDistalSynapse = self.selectedDistalSegment.synapses[index]
      self.selectedDistalSynapse.tree3d_selected = True

    Global.simulationForm.refreshControls()

  #endregion



import sys
import time
import webbrowser
from PyQt4 import QtGui, QtCore
from nupic_studio import MachineState
from nupic_studio.htm import maxPreviousSteps, maxPreviousStepsWithInference
from nupic_studio.ui import Global
from nupic_studio.ui.project_properties_form import ProjectPropertiesForm

class MainForm(QtGui.QMainWindow):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QMainWindow.__init__(self)

    #region  Instance fields

    self._pendingProjectChanges = False
    self._numStepsPending = 0

    #endregion

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # menuFileNew
    self.menuFileNew = QtGui.QAction(self)
    self.menuFileNew.setText("&New Project")
    self.menuFileNew.setShortcut('Ctrl+N')
    self.menuFileNew.triggered.connect(self.newProject)

    # menuFileOpen
    self.menuFileOpen = QtGui.QAction(self)
    self.menuFileOpen.setText("&Open  Project")
    self.menuFileOpen.setShortcut('Ctrl+O')
    self.menuFileOpen.triggered.connect(self.openProject)

    # menuFileSave
    self.menuFileSave = QtGui.QAction(self)
    self.menuFileSave.setText("&Save Project")
    self.menuFileSave.setShortcut('Ctrl+S')
    self.menuFileSave.triggered.connect(self.saveProject)

    # menuFileExit
    self.menuFileExit = QtGui.QAction(self)
    self.menuFileExit.setText("&Exit")
    self.menuFileExit.setShortcut('Ctrl+Q')
    self.menuFileExit.triggered.connect(self.__menuFileExit_Click)

    # menuFile
    self.menuFile = QtGui.QMenu()
    self.menuFile.addAction(self.menuFileNew)
    self.menuFile.addAction(self.menuFileOpen)
    self.menuFile.addAction(self.menuFileSave)
    self.menuFile.addSeparator()
    self.menuFile.addAction(self.menuFileExit)
    self.menuFile.setTitle("&File")

    # menuViewArchitecture
    self.menuViewArchitecture = QtGui.QAction(self)
    self.menuViewArchitecture.setText("&Network Architecture")
    self.menuViewArchitecture.triggered.connect(self.__menuViewArchitecture_Click)

    # menuViewSimulation
    self.menuViewSimulation = QtGui.QAction(self)
    self.menuViewSimulation.setText("&Simulation")
    self.menuViewSimulation.triggered.connect(self.__menuViewSimulation_Click)

    # menuViewNodeInformation
    self.menuViewNodeInformation = QtGui.QAction(self)
    self.menuViewNodeInformation.setText("Node &Information")
    self.menuViewNodeInformation.triggered.connect(self.__menuViewNodeInformation_Click)

    # menuViewOutput
    self.menuViewOutput = QtGui.QAction(self)
    self.menuViewOutput.setText("&Output")
    self.menuViewOutput.triggered.connect(self.__menuViewOutput_Click)

    # menuViewToolWindows
    self.menuViewToolWindows = QtGui.QMenu()
    self.menuViewToolWindows.addAction(self.menuViewArchitecture)
    self.menuViewToolWindows.addAction(self.menuViewNodeInformation)
    self.menuViewToolWindows.addAction(self.menuViewSimulation)
    self.menuViewToolWindows.addAction(self.menuViewOutput)
    self.menuViewToolWindows.setTitle("Tool &Windows")

    # menuView
    self.menuView = QtGui.QMenu()
    self.menuView.addMenu(self.menuViewToolWindows)
    self.menuView.setTitle("&View")

    # menuEdit
    self.menuEdit = QtGui.QMenu()
    self.menuEdit.setTitle("&Edit")

    # menuProjectProperties
    self.menuProjectProperties = QtGui.QAction(self)
    self.menuProjectProperties.setText("Properties...")
    self.menuProjectProperties.triggered.connect(self.__menuProjectProperties_Click)

    # menuProject
    self.menuProject = QtGui.QMenu()
    self.menuProject.addAction(self.menuProjectProperties)
    self.menuProject.setTitle("&Project")

    # menuTools
    self.menuTools = QtGui.QMenu()
    self.menuTools.setTitle("&Tools")

    # menuUserWiki
    self.menuUserWiki = QtGui.QAction(self)
    self.menuUserWiki.setText("User Wiki")
    self.menuUserWiki.triggered.connect(self.__menuUserWiki_Click)

    # menuGoToWebsite
    self.menuGoToWebsite = QtGui.QAction(self)
    self.menuGoToWebsite.setText("Project Website")
    self.menuGoToWebsite.triggered.connect(self.__menuGoToWebsite_Click)

    # menuAbout
    self.menuAbout = QtGui.QAction(self)
    self.menuAbout.setText("About")
    self.menuAbout.triggered.connect(self.__menuAbout_Click)

    # menuHelp
    self.menuHelp = QtGui.QMenu()
    self.menuHelp.addAction(self.menuUserWiki)
    self.menuHelp.addAction(self.menuGoToWebsite)
    self.menuHelp.addAction(self.menuAbout)
    self.menuHelp.setTitle("&Help")

    # menuMain
    self.menuMain = self.menuBar()
    self.menuMain.addMenu(self.menuFile)
    self.menuMain.addMenu(self.menuView)
    self.menuMain.addMenu(self.menuProject)
    self.menuMain.addMenu(self.menuHelp)

    # buttonInitHTM
    self.buttonInitHTM = QtGui.QAction(self)
    self.buttonInitHTM.setEnabled(False)
    self.buttonInitHTM.setIcon(QtGui.QIcon(Global.appPath + '/images/buttonInitializeHTM.png'))
    self.buttonInitHTM.setToolTip("Initialize simulation")
    self.buttonInitHTM.triggered.connect(self.__buttonInitHTM_Click)

    # buttonStepHTM
    self.buttonStepHTM = QtGui.QAction(self)
    self.buttonStepHTM.setEnabled(False)
    self.buttonStepHTM.setIcon(QtGui.QIcon(Global.appPath + '/images/buttonStepHTM.png'))
    self.buttonStepHTM.setToolTip("Forward one time step")
    self.buttonStepHTM.triggered.connect(self.__buttonStepHTM_Click)

    # buttonMultipleStepsHTM
    self.buttonMultipleStepsHTM = QtGui.QAction(self)
    self.buttonMultipleStepsHTM.setEnabled(False)
    self.buttonMultipleStepsHTM.setIcon(QtGui.QIcon(Global.appPath + '/images/buttonStepFastHTM.png'))
    self.buttonMultipleStepsHTM.setToolTip("Forward a specific number of time steps")
    self.buttonMultipleStepsHTM.triggered.connect(self.__buttonMultipleStepsHTM_Click)

    # buttonStopHTM
    self.buttonStopHTM = QtGui.QAction(self)
    self.buttonStopHTM.setEnabled(False)
    self.buttonStopHTM.setIcon(QtGui.QIcon(Global.appPath + '/images/buttonStopHTM.png'))
    self.buttonStopHTM.setToolTip("Stop simulation")
    self.buttonStopHTM.triggered.connect(self.__buttonStopHTM_Click)

    # textBoxStep
    self.textBoxStep = QtGui.QLineEdit()
    self.textBoxStep.setEnabled(False)
    self.textBoxStep.setAlignment(QtCore.Qt.AlignRight)
    self.textBoxStep.setFixedSize(QtCore.QSize(80, 20))

    # sliderStep
    self.sliderStep = QtGui.QSlider()
    self.sliderStep.setEnabled(False)
    self.sliderStep.setOrientation(QtCore.Qt.Horizontal)
    self.sliderStep.setSingleStep(1)
    self.sliderStep.setRange(0, maxPreviousSteps - 1)
    self.sliderStep.setValue(maxPreviousSteps - 1)
    self.sliderStep.valueChanged.connect(self.__sliderStep_ValueChanged)

    # toolBar
    self.toolBar = QtGui.QToolBar()
    self.toolBar.addAction(self.buttonInitHTM)
    self.toolBar.addAction(self.buttonStepHTM)
    self.toolBar.addAction(self.buttonMultipleStepsHTM)
    self.toolBar.addAction(self.buttonStopHTM)
    self.toolBar.addWidget(self.textBoxStep)
    self.toolBar.addWidget(self.sliderStep)

    # dockArchitectureForm
    self.dockArchitectureForm = QtGui.QDockWidget()
    self.dockArchitectureForm.setWidget(Global.architectureForm)
    self.dockArchitectureForm.setWindowTitle(Global.architectureForm.windowTitle())

    # dockSimulationForm
    self.dockSimulationForm = QtGui.QDockWidget()
    self.dockSimulationForm.setWidget(Global.simulationForm)
    self.dockSimulationForm.setWindowTitle(Global.simulationForm.windowTitle())

    # dockNodeInformationForm
    self.dockNodeInformationForm = QtGui.QDockWidget()
    self.dockNodeInformationForm.setWidget(Global.nodeInformationForm)
    self.dockNodeInformationForm.setWindowTitle(Global.nodeInformationForm.windowTitle())

    # dockOutputForm
    self.dockOutputForm = QtGui.QDockWidget()
    self.dockOutputForm.setWidget(Global.outputForm)
    self.dockOutputForm.setWindowTitle(Global.outputForm.windowTitle())

    # MainForm
    self.addToolBar(self.toolBar)
    self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.dockArchitectureForm)
    self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.dockSimulationForm)
    self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.dockNodeInformationForm)
    self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.dockOutputForm)
    self.tabifyDockWidget(self.dockNodeInformationForm, self.dockOutputForm)
    self.setCentralWidget(self.dockSimulationForm)
    self.setWindowTitle("NuPIC Studio")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))

  def __cleanUp(self):
    """
    Prepare UI to load a new configuration.
    """

    self.__enableSimulationButtons(False)
    self.__enableSteeringButtons(False)

    # Update architecture controls
    Global.architectureForm.designPanel.topRegion = Global.project.network.nodes[0]
    Global.architectureForm.designPanel.selectedNode = Global.project.network.nodes[0]
    Global.architectureForm.designPanel.repaint()
    Global.architectureForm.updateCode()

    # Reset the controls
    self.clearControls()

  def __enableSimulationButtons(self, enable):
    """
    Enables or disables controls related to simulation.
    """

    Global.simulationInitialized = enable
    self.buttonInitHTM.setEnabled(not enable)
    if not enable:
      self.textBoxStep.setText("")
      self.sliderStep.setEnabled(False)
      self.sliderStep.setValue(self.sliderStep.maximum())

  def __enableSteeringButtons(self, enable):
    """
    Enables or disables buttons in toolbar.
    """

    self.buttonStepHTM.setEnabled(enable)
    self.buttonMultipleStepsHTM.setEnabled(enable)
    self.buttonStopHTM.setEnabled(enable)

  def clearControls(self):
    """
    Reset the controls.
    """

    Global.simulationForm.clearControls()
    Global.outputForm.clearControls()
    Global.nodeInformationForm.clearControls()

  def refreshControls(self):
    """
    Refresh controls for each time step.
    """

    if Global.simulationInitialized :
      maxTime = Global.currStep + 1
      selStep = maxTime - (self.sliderStep.maximum() - self.sliderStep.value())
      self.textBoxStep.setText(str(selStep) + "/" + str(maxTime))
    else:
      self.textBoxStep.setText("")

    if self.dockSimulationForm.isVisible():
      Global.simulationForm.refreshControls()
    if self.dockNodeInformationForm.isVisible():
      Global.nodeInformationForm.refreshControls()

  def markProjectChanges(self, hasChanges):
    """
    Provides an UI reaction to any project changes or a new or saved unchanged project.
    """

    self._pendingProjectChanges = hasChanges
    self.menuFileSave.setEnabled(hasChanges)

  def __checkCurrentConfigChanges(self):
    """
    Checks if the current file has changed.
    """

    result = QtGui.QMessageBox.No

    # If changes happened, ask to user if he wish saves them
    if self._pendingProjectChanges:
      result = QtGui.QMessageBox.question(self, "Question", "Current project has changed. Do you want save these changes?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No, QtGui.QMessageBox.Cancel)
      if result == QtGui.QMessageBox.Yes:
        self.saveProject()

    return result

  def newProject(self):
    """
    Creates a new project.
    """

    # Check if the current project has changed before continue operation
    if self.__checkCurrentConfigChanges() != QtGui.QMessageBox.Cancel:
      # Create new project
      Global.project.new()

      # Initialize project state
      self.setWindowTitle(Global.project.name + " - NuPIC Studio")
      self.markProjectChanges(False)
      self.__cleanUp()

      return True

    return False

  def openProject(self):
    """
    Open an existing project
    """

    # Check if the current project has changed before continue operation
    if self.__checkCurrentConfigChanges() != QtGui.QMessageBox.Cancel:

      # Ask user for an existing file
      selectedFile = str(QtGui.QFileDialog().getOpenFileName(self, "Open Project", Global.appPath + '/projects', "NuPIC project files (*.nuproj)"))

      # If file exists, continue operation
      if selectedFile != '':
        # Open the selected project
        Global.project.open(selectedFile)

        # Initialize project state
        self.setWindowTitle(Global.project.name + " - [" + Global.project.fileName + "] - NuPIC Studio")
        self.markProjectChanges(False)
        self.__cleanUp()

        return True

    return False

  def saveProject(self):
    """
    Save the current project
    """

    # If current project is new, ask user for valid file
    fileName = Global.project.fileName
    if fileName == '':
      # Ask user for valid file
      selectedFile = str(QtGui.QFileDialog().getSaveFileName(self, "Save Project", Global.appPath + '/projects', "NuPIC project files (*.nuproj)"))

      # If file exists, continue operation
      if selectedFile != '':
        fileName = selectedFile

    # If file is Ok, continue operation
    if fileName != '':
      # Save to the selected location
      Global.project.save(fileName)

      # Initialize project state
      self.setWindowTitle(Global.project.name + " - [" + Global.project.fileName + "] - NuPIC Studio")
      self.markProjectChanges(False)

      return True

    return False

  def stopSimulation(self):

    # Disable relevant buttons to reset
    self.__enableSteeringButtons(False)
    self.__enableSimulationButtons(False)

    # Reset controls
    self.clearControls()

  #endregion

  #region Events

  #region Form

  def closeEvent(self, event):
    if self.__checkCurrentConfigChanges() == QtGui.QMessageBox.Cancel:
      event.ignore()
    else:
      if self.buttonStopHTM.isEnabled():
        self.stopSimulation()
      sys.exit()

  #endregion

  #region Menus

  def __menuFileExit_Click(self, event):
    self.close()

  def __menuProjectProperties_Click(self, event):
    # Open Project properties form
    projectPropertiesForm = ProjectPropertiesForm()
    projectPropertiesForm.setControlsValues()
    dialogResult = projectPropertiesForm.exec_()

    if dialogResult == QtGui.QDialog.Accepted:
      Global.mainForm.markProjectChanges(True)

  def __menuViewArchitecture_Click(self, event):
    self.dockArchitectureForm.show()

  def __menuViewSimulation_Click(self, event):
    self.dockSimulationForm.show()
    Global.simulationForm.refreshControls()

  def __menuViewNodeInformation_Click(self, event):
    self.dockNodeInformationForm.show()
    Global.nodeInformationForm.refreshControls()

  def __menuViewOutput_Click(self, event):
    self.dockOutputForm.show()

  def __menuUserWiki_Click(self, event):
    webbrowser.open('https://github.com/nupic-community/nupic.studio/wiki')

  def __menuGoToWebsite_Click(self, event):
    webbrowser.open('https://github.com/nupic-community/nupic.studio')

  def __menuAbout_Click(self, event):
    QtGui.QMessageBox.information(self, "Information", "v. " + Global.version + "\nGet more info at our home page.")

  #endregion

  #region Toolbar

  def __buttonInitHTM_Click(self, event):
    """
    Initializes the HTM-Network by creating the htm-controller to connect to events database
    """

    # Initialize the network starting from top region.
    startTime = time.time()
    endTime = time.time()
    initialized = Global.project.network.initialize()

    if initialized:

      # Initialize time steps parameters
      Global.currStep = 0
      Global.selStep = 0
      Global.timeStepsPredictionsChart = MachineState(0, maxPreviousStepsWithInference)

      Global.outputForm.addText("Initialization: " + "{0:.3f}".format(endTime - startTime) + " secs")
      Global.outputForm.addText("")
      Global.outputForm.addText("Step\tTime (secs)\tAccuracy (%)")

      # Perfoms actions related to time step progression.
      startTime = time.time()
      Global.project.network.nextStep()
      Global.project.network.calculateStatistics()
      endTime = time.time()
      Global.outputForm.addText(str(Global.currStep + 1) + "\t{0:.3f}".format(endTime - startTime) + "\t{0:.3f}".format(Global.project.network.statsPrecisionRate))

      # Disable relevant buttons:
      self.__enableSteeringButtons(True)
      self.__enableSimulationButtons(True)

      # Update controls
      Global.simulationForm.topRegion = Global.project.network.nodes[0]
      Global.simulationForm.initializeControls()
      self.refreshControls()

  def __buttonStepHTM_Click(self, event):
    """
    Performs a single simulation step.
    """

    # Update time steps parameters
    Global.currStep += 1
    if Global.currStep >= (maxPreviousSteps - 1):
      self.sliderStep.setEnabled(True)
    Global.selStep = self.sliderStep.maximum() - self.sliderStep.value()
    Global.timeStepsPredictionsChart.rotate()
    Global.timeStepsPredictionsChart.setForCurrStep(Global.currStep)

    # Perfoms actions related to time step progression.
    startTime = time.time()
    Global.project.network.nextStep()
    Global.project.network.calculateStatistics()
    endTime = time.time()
    Global.outputForm.addText(str(Global.currStep + 1) + "\t{0:.3f}".format(endTime - startTime) + "\t{0:.3f}".format(Global.project.network.statsPrecisionRate))

    # Update controls
    self.refreshControls()

  def __buttonMultipleStepsHTM_Click(self, event):
    """
    Performs full HTM simulation.
    """

    # Get number of steps to perform simulation
    self._numStepsPending = -1
    enteredInteger, ok = QtGui.QInputDialog.getInt(self, "Input Dialog", "Enter number of steps:")
    if ok:
      if enteredInteger < 2:
        QtGui.QMessageBox.warning(self, "Warning", "Invalid value specified!")
      else:
        self._numStepsPending = enteredInteger

    while self._numStepsPending > 0:
      self.__buttonStepHTM_Click(event)
      Global.app.processEvents()
      self._numStepsPending -= 1

  def __buttonStopHTM_Click(self, event):
    # If multiple steps processing is running just stop the loop
    # otherwise, ask user to stop the simulation
    if self._numStepsPending > 0:
      self._numStepsPending = 0
    else:
      dialogResult = QtGui.QMessageBox.question(self, "Question", "Current simulation (learning) will stop!\r\nDo you want proceed?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
      if dialogResult == QtGui.QMessageBox.Yes:
        self.stopSimulation()

  def __sliderStep_ValueChanged(self, value):
    Global.selStep = Global.selStep = self.sliderStep.maximum() - self.sliderStep.value()
    self.refreshControls()

  #endregion

  #endregion



import collections
import json
import pkg_resources

__version__ = "1.1.3"


def versionList(versionString):
  """
  Transform a version from string to integer list in order to make possible versions comparison.

  :param versionString: a string containing the version in the format '9.9.9.xxx'
  :return: a integer list containing the version in the format ['9', '9', '9']. Alphanumerics are ignored.
  """

  versionInt = []
  versionSplit = versionString.split(".")
  for v in versionSplit:
    if v.isdigit():
      versionInt.append(v)
    else:
      break
  return versionInt

try:
  import nupic
except ImportError:
  raise ImportError("NuPIC library not found! Access https://github.com/numenta/nupic/ for get help on how install it.")
foundNupic = pkg_resources.get_distribution("nupic")
versionRequiredMin = "0.2.2"
versionRequiredMax = "99.99.99"
if not (versionList(versionRequiredMin) <= versionList(foundNupic.version) <= versionList(versionRequiredMax)):
  raise Exception("Unexpected version of NuPIC Library! Expected between %s and %s, but detected %s in %s." % (versionRequiredMin, versionRequiredMax, foundNupic.version, foundNupic.location))

try:
  from PyQt4 import Qt, QtCore, QtGui, QtOpenGL
except ImportError:
  msg = "PyQt4 library not found! Access http://pyqt.sourceforge.net/Docs/PyQt4/installation.html for get help on how install it" \
    "...or use a package manager like apt, yum, or brew:\n" \
    "  apt-get install python-qt4 python-qt4-gl\n" \
    "  yum install PyQt4\n" \
    "  brew install pyqt"
  raise ImportError(msg)

class MachineState(object):
  """
  This class consists of a queue with max length to store states for each time step.
  """

  def __init__(self, defaultValue, maxLen):
    self.defaultValue = defaultValue
    self.maxLen = maxLen
    self.list = [defaultValue] * maxLen

  def getList(self):
    """
    Get list with stored states machine.
    """

    return self.list

  def rotate(self):
    """
    Update states machine by remove the first element and add a new element in the end.
    """

    self.list.remove(self.list[0])
    self.list.append(self.defaultValue)

  def atGivenStepAgo(self, timeStep):
    """
    Get the state for a given time step.
    """

    return self.list[len(self.list) - timeStep - 1]

  def setForCurrStep(self, value):
    """
    Set the state for the current time step.
    """

    self.list[len(self.list) - 1] = value

  def atCurrStep(self):
    """
    Get the state of the current time step.
    """

    return self.list[len(self.list) - 1]

  def atPreviousStep(self):
    """
    Get the state of the previous time step.
    """

    return self.list[len(self.list) - 2]

  def atFirstStep(self):
    """
    Get the state of the firt time step.
    """

    return self.list[0]

def getInstantiatedClass(moduleName, className, classParams):
  """
  Return an instantiated class given a module, class, and constructor params
  """

  # Remove single quote from parameter values
  #   foo: 'bar' => foo: bar
  classParams = classParams.replace(": '", ": ")
  classParams = classParams.replace("', ", ", ")
  classParams = classParams.replace("'}", "}")
  classParams = classParams.replace("'", "\"")

  module = __import__(moduleName, fromlist=[className])
  class_ = getattr(module, className)
  params = json.loads(classParams, object_pairs_hook=collections.OrderedDict)
  instance = class_(**params)

  return instance

class ArrayTableModel(QtGui.QStandardItemModel):

  def __init__(self, flags):
    QtGui.QStandardItemModel.__init__(self)

    self.flags = flags
    self.header = []
    self.data = []

  def update(self, header, data):
    self.header = header
    self.data = data

    numCols = len(self.header)
    self.setColumnCount(numCols)
    numRows = len(self.data)
    self.setRowCount(numRows)

    for col in range(numCols):
      self.setHeaderData(col, QtCore.Qt.Horizontal, self.header[col])

    for row in range(numRows):
      for col in range(numCols):
        value = self.data[row][col]
        self.setData(self.index(row, col, QtCore.QModelIndex()), value)

  def setData(self, index, value, role=None):
    self.data[index.row()][index.column()] = value
    self.dataChanged.emit(index, index)
    return True

  def data(self, index, role=None):
    column, row = index.column(), index.row()
    if role == QtCore.Qt.TextAlignmentRole:
      return QtCore.Qt.AlignRight
    elif role == QtCore.Qt.DisplayRole:
      return self.data[row][column]
    return

  def columnCount(self, parent=None, **kwargs):
    return len(self.header)

  def rowCount(self, parent=None, **kwargs):
    return len(self.data)

  def flags(self, index):
    return self.flags



from PyQt4 import QtGui, QtCore
from nupic_studio.ui import Global
from nupic_studio.htm.node import NodeType, Node
from nupic_studio.htm.node_region import Region
from nupic_studio.htm.node_sensor import Sensor
from nupic_studio.ui.node_region_form import RegionForm
from nupic_studio.ui.node_sensor_form import SensorForm

class ArchitectureForm(QtGui.QWidget):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QWidget.__init__(self)

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # DesignPanel
    self.designPanel = DesignPanel()

    # tabPageDesignLayout
    tabPageDesignLayout = QtGui.QHBoxLayout()
    tabPageDesignLayout.addWidget(self.designPanel)

    # tabPageDesign
    self.tabPageDesign = QtGui.QWidget()
    self.tabPageDesign.setLayout(tabPageDesignLayout)

    # textBoxCode
    self.textBoxCode = QtGui.QTextEdit()
    self.textBoxCode.setReadOnly(True)
    self.textBoxCode.setAlignment(QtCore.Qt.AlignLeft)
    self.textBoxCode.setWordWrapMode(QtGui.QTextOption.NoWrap)
    self.textBoxCode.setFont(QtGui.QFont("Courier New", 9))

    # tabPageCodeLayout
    tabPageCodeLayout = QtGui.QHBoxLayout()
    tabPageCodeLayout.addWidget(self.textBoxCode)

    # tabPageCode
    self.tabPageCode = QtGui.QWidget()
    self.tabPageCode.setLayout(tabPageCodeLayout)

    # tabControlMain
    self.tabControlMain = QtGui.QTabWidget()
    self.tabControlMain.addTab(self.tabPageDesign, "Design")
    self.tabControlMain.addTab(self.tabPageCode, "Code")

    # layout
    layout = QtGui.QHBoxLayout()
    layout.addWidget(self.tabControlMain)

    # ArchitectureForm
    self.setLayout(layout)
    self.setMinimumWidth(300)
    self.setMinimumHeight(300)
    self.setWindowTitle("Network Architecture")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))

  def updateCode(self):
    """
    Update the source code of the network.
    """

    code = Global.project.network.getSourceCode()
    self.textBoxCode.setText(code)

  #endregion

class DesignPanel(QtGui.QWidget):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QWidget.__init__(self)

    #region Instance fields

    self.topRegion = Region("TopRegion")
    """Node that is on top of the hierarchy."""

    self.selectedNode = None
    """Node that is selected for visualization of its details."""

    self.underMouseNode = None
    """Node that is highlighted due to mouse is on it."""

    # Space to skip horizontally between siblings
    # and vertically between generations
    self._offsetHorizontal = 15
    self._offsetVertical = 30

    #endregion

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # menuNodeProperties
    self.menuNodeProperties = QtGui.QAction(self)
    self.menuNodeProperties.setText("&Properties")
    self.menuNodeProperties.triggered.connect(self.__menuNodeProperties_Click)

    # menuNodeAddRegion
    self.menuNodeAddRegion = QtGui.QAction(self)
    self.menuNodeAddRegion.setText("&Add region under this node...")
    self.menuNodeAddRegion.triggered.connect(self.__menuNodeAddRegion_Click)

    # menuNodeAddSensor
    self.menuNodeAddSensor = QtGui.QAction(self)
    self.menuNodeAddSensor.setText("&Add sensor under this node...")
    self.menuNodeAddSensor.triggered.connect(self.__menuNodeAddSensor_Click)

    # menuNodeDelete
    self.menuNodeDelete = QtGui.QAction(self)
    self.menuNodeDelete.setText("&Delete this node...")
    self.menuNodeDelete.triggered.connect(self.__menuNodeDelete_Click)

    # menuNode
    self.menuNode = QtGui.QMenu()
    self.menuNode.addAction(self.menuNodeProperties)
    self.menuNode.addAction(self.menuNodeAddRegion)
    self.menuNode.addAction(self.menuNodeAddSensor)
    self.menuNode.addAction(self.menuNodeDelete)

    # layout
    layout = QtGui.QHBoxLayout()

    # DesignPanel
    self.setLayout(layout)
    self.setToolTip("Left button click: Select region.\r\nRight button click: Show options for region or sensor.")

    # Set center position of the top region
    self.topRegion.tree2d_x = self.minimumWidth() / 2
    self.topRegion.tree2d_y = 30

    # Painter to draw the tree
    self.painter = QtGui.QPainter()

  def __arrangeNode(self, node, minX, minY):
    """
    Arrange the node and the lower nodes that feed it in the allowed area.
    Set minX to indicate the right edge of our subtree.
    Set minY to indicate the bottom edge of our subtree.
    """

    # See how big this node is.
    size = self.__getNodeSize(node)

    # Recursively arrange the lower nodes that feed this node,
    # allowing room for this node.
    x = minX
    width = size.width()
    height = size.height()
    biggestMinY = minY + height
    subtreeMinY = minY + height + self._offsetVertical
    numFeeders = 0
    for feeder in Global.project.network.getFeederNodes(node):

      # Arrange this feeder's subtree.
      feederMinY = subtreeMinY
      x, feederMinY = self.__arrangeNode(feeder, x, feederMinY)

      # See if this increases the biggest minY value.
      if biggestMinY < feederMinY:
        biggestMinY = feederMinY

      # Allow room before the next sibling.
      x += self._offsetHorizontal

      numFeeders += 1

    # Remove the spacing after the last feeder.
    if numFeeders > 0:
      x -= self._offsetHorizontal

    # See if this node is wider than the subtree under it.
    subtreeWidth = x - minX
    if width > subtreeWidth:
      # Center the subtree under this node.
      # Make the lower nodes that feed this node rearrange themselves
      # moved to center their subtrees.
      x = minX + (width - subtreeWidth) / 2
      for feeder in Global.project.network.getFeederNodes(node):
        # Arrange this feeder's subtree.
        x, subtreeMinY = self.__arrangeNode(feeder, x, subtreeMinY)

        # Allow room before the next sibling.
        x += self._offsetHorizontal

      # The subtree's width is this node's width.
      subtreeWidth = width

    # Set this node's center position.
    node.tree2d_x = minX + subtreeWidth / 2
    node.tree2d_y = minY + height / 2

    # Increase minX to allow room for the subtree before returning.
    minX += subtreeWidth

    # Set the return value for minY.
    minY = biggestMinY

    return minX, minY

  def __getNodeSize(self, node):
    """
    Return the size of the string plus a 10 pixel margin.
    """

    fontMetrics = self.painter.fontMetrics()
    width = fontMetrics.width(QtCore.QString(node.name))

    return QtCore.QSizeF(30 + width, 30)

  def __drawNode(self, node):
    """
    Draw the nodes for the subtree rooted at this node.
    """

    # Recursively make the node draw its feeders.
    for feeder in Global.project.network.getFeederNodes(node):
      # Draw the link between this node and this feeder.
      self.painter.drawLine(node.tree2d_x, node.tree2d_y, feeder.tree2d_x, feeder.tree2d_y)

      # Recursively make the node draw its feeders.
      self.__drawNode(feeder)

    # Draw this node centered at (x, y).
    brush = QtGui.QBrush()
    if node == self.selectedNode:
      brush = QtGui.QColor(0, 200, 250)
    else:
      brush = QtGui.QColor(0, 150, 200)

    # Fill and draw a polygon at our location.
    size = self.__getNodeSize(node)
    x = node.tree2d_x
    y = node.tree2d_y
    width = size.width()
    height = size.height()
    if node.type == NodeType.region:
      point1 = QtCore.QPoint((x - width / 2) + 10, (y - height / 2))
      point2 = QtCore.QPoint((x + width / 2) - 10, (y - height / 2))
    elif (node.type == NodeType.sensor):
      point1 = QtCore.QPoint((x - width / 2), (y - height / 2))
      point2 = QtCore.QPoint((x + width / 2), (y - height / 2))
    point3 = QtCore.QPoint((x + width / 2), (y + height / 2))
    point4 = QtCore.QPoint((x - width / 2), (y + height / 2))
    polygon = QtGui.QPolygon([point1, point2, point3, point4])
    self.painter.setBrush(brush)
    self.painter.drawPolygon(polygon)
    node.tree2d_polygon = polygon

    # Draw the text.
    self.painter.drawText(polygon.boundingRect(), QtCore.Qt.AlignCenter, node.name)

  def __nodeAtPoint(self, node, mousePoint):
    """
    Return the node at this point (or None if there isn't one there).
    """

    # See if the point is under this node.
    if node.tree2d_polygon.boundingRect().contains(mousePoint):
      return node

    # See if the point is under a node in the subtree.
    for feeder in Global.project.network.getFeederNodes(node):
      hitNode = self.__nodeAtPoint(feeder, mousePoint)
      if hitNode != None:
        return hitNode

    return None

  #endregion

  #region Events

  def paintEvent(self, event):
    """
    Draw and center the tree on the form.
    """

    # Initialize painter
    self.painter.begin(self)
    self.painter.setFont(QtGui.QFont("Arial", 8))
    self.painter.fillRect(self.rect(), QtCore.Qt.white)

    # Arrange the tree once to see how big it is.
    minX = 0
    minY = 0
    minX, minY = self.__arrangeNode(self.topRegion, minX, minY)

    # Rearrange the tree again to center it horizontally.
    minX = (self.width() - minX) / 2
    minY = 10
    minX, minY = self.__arrangeNode(self.topRegion, minX, minY)

    # Draw the tree recursively from top region.
    self.__drawNode(self.topRegion)

    # End painter
    self.painter.end()

  def mousePressEvent(self, event):
    """
    If this is a right button down and the mouse is over a node, display a context menu.
    """

    if event.buttons() == QtCore.Qt.LeftButton:
      self.underMouseNode = self.__nodeAtPoint(self.topRegion, event.pos())
      if self.underMouseNode != None:
        # Select the node and updates any related information.
        self.selectedNode = self.underMouseNode

        # Redraw the tree to show the updates.
        self.repaint()

        # Refresh dependents tools
        Global.simulationForm.refreshControls()
        Global.nodeInformationForm.refreshControls()
    elif event.buttons() == QtCore.Qt.RightButton:
      self.underMouseNode = self.__nodeAtPoint(self.topRegion, event.pos())
      if self.underMouseNode != None:
        # Don't let the user delete the top node.
        self.menuNodeAddRegion.setEnabled(not Global.simulationInitialized and self.underMouseNode.type != NodeType.sensor)
        self.menuNodeAddSensor.setEnabled(not Global.simulationInitialized and self.underMouseNode.type != NodeType.sensor)
        self.menuNodeDelete.setEnabled(not Global.simulationInitialized and self.underMouseNode != self.topRegion)

        # Display the context menu.
        self.menuNode.exec_(self.mapToGlobal(event.pos()))

  def __menuNodeProperties_Click(self, event):
    """
    View node propeerties.
    """

    if self.underMouseNode.type == NodeType.region:
      regionForm = RegionForm()
      regionForm.setControlsValues()
      dialogResult = regionForm.exec_()

      # Update controls with the new changes
      if dialogResult == QtGui.QDialog.Accepted:
        Global.mainForm.markProjectChanges(True)
        Global.architectureForm.updateCode()
    elif self.underMouseNode.type == NodeType.sensor:
      sensorForm = SensorForm()
      sensorForm.setControlsValues()
      dialogResult = sensorForm.exec_()

      # Update controls with the new changes
      if dialogResult == QtGui.QDialog.Accepted:
        Global.mainForm.markProjectChanges(True)
        Global.architectureForm.updateCode()

  def __menuNodeAddRegion_Click(self, event):
    """
    Add a feeder region to the selected region.
    """

    # Ask for region's name
    enteredText, ok = QtGui.QInputDialog.getText(self, "Input Dialog", "Enter region's name:")
    if ok:
      validExpr = QtCore.QRegExp('[a-zA-Z0-9_]+')
      if not validExpr.exactMatch(enteredText):
        QtGui.QMessageBox.warning(self, "Warning", "'" + enteredText + "' is not a valid name. Only characters, numbers and _ are accepted.")
        return

      Global.mainForm.markProjectChanges(True)

      # Add new region below highlighted region
      newRegion = Region(enteredText)
      Global.project.network.addFeederNode(newRegion, self.underMouseNode)

      # Redraw the tree to show the updates.
      self.repaint()
      Global.architectureForm.updateCode()

  def __menuNodeAddSensor_Click(self, event):
    """
    Add a feeder sensor to the selected region.
    """

    # Ask for sensor's name
    enteredText, ok = QtGui.QInputDialog.getText(self, "Input Dialog", "Enter sensor's name:")
    if ok:
      validExpr = QtCore.QRegExp('[a-zA-Z0-9_]+')
      if not validExpr.exactMatch(enteredText):
        QtGui.QMessageBox.warning(self, "Warning", "'" + enteredText + "' is not a valid name. Only characters, numbers and _ are accepted.")
        return

      Global.mainForm.markProjectChanges(True)

      # Add new sensor below highlighted region
      newSensor = Sensor(enteredText)
      Global.project.network.addFeederNode(newSensor, self.underMouseNode)

      # Redraw the tree to show the updates.
      self.repaint()
      Global.architectureForm.updateCode()

  def __menuNodeDelete_Click(self, event):
    """
    Delete this node from the tree.
    """

    if QtGui.QMessageBox.question(self, "Question", "Are you sure you want to delete this node?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
      Global.mainForm.markProjectChanges(True)

      # Delete the node and its subtree.
      Global.project.network.deleteFeederNode(self.underMouseNode)

      # Redraw the tree to show the updates.
      self.repaint()
      Global.architectureForm.updateCode()

  #endregion



from nupic_studio import MachineState
from nupic_studio.htm import maxPreviousSteps
from nupic_studio.ui import Global

class Bit:
  """
  A class only to group properties related to input bits of sensors.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    self.initialize()

  #endregion

  #region Methods

  def initialize(self):
    """
    Initialize this bit.
    """

    self.x = -1
    """Position on X axis"""

    self.y = -1
    """Position on Y axis"""

    # States of this element
    self.isActive = MachineState(False, maxPreviousSteps)
    self.isPredicted = MachineState(False, maxPreviousSteps)
    self.isFalselyPredicted = MachineState(False, maxPreviousSteps)

    #region Statistics properties

    self.statsActivationCount = 0
    self.statsActivationRate = 0.
    self.statsPreditionCount = 0
    self.statsPrecisionRate = 0.

    #endregion

    #region 3d-tree properties (simulation form)

    self.tree3d_initialized = False
    self.tree3d_x = 0
    self.tree3d_y = 0
    self.tree3d_z = 0
    self.tree3d_item = None
    self.tree3d_selected = False

    #endregion

  def nextStep(self):
    """
    Perfoms actions related to time step progression.
    """

    # Update states machine by remove the first element and add a new element in the end
    self.isActive.rotate()
    self.isPredicted.rotate()
    self.isFalselyPredicted.rotate()

  def calculateStatistics(self):
    """
    Calculate statistics after an iteration.
    """

    # Calculate statistics
    if self.isActive.atCurrStep():
      self.statsActivationCount += 1
    if self.isPredicted.atCurrStep():
      self.statsPreditionCount += 1
    if Global.currStep > 0:
      self.statsActivationRate = self.statsActivationCount / float(Global.currStep)
    if self.statsActivationCount > 0:
      self.statsPrecisionRate = self.statsPreditionCount / float(self.statsActivationCount)

  #endregion



from PyQt4 import QtGui, QtCore
from nupic_studio.ui import Global

class SimulationLegendForm(QtGui.QDialog):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QWidget.__init__(self)

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # buttonInactiveElement
    self.buttonInactiveElement = QtGui.QPushButton()
    self.buttonInactiveElement.setStyleSheet("background-color: rgb(190, 190, 190)")
    self.buttonInactiveElement.setEnabled(False)

    # labelInactiveElement
    self.labelInactiveElement = QtGui.QLabel()
    self.labelInactiveElement.setText("Inactive")

    # buttonFalselyPredictedElement
    self.buttonFalselyPredictedElement = QtGui.QPushButton()
    self.buttonFalselyPredictedElement.setStyleSheet("background-color: rgb(255, 0, 0)")
    self.buttonFalselyPredictedElement.setEnabled(False)

    # labelFalselyPredictedElement
    self.labelFalselyPredictedElement = QtGui.QLabel()
    self.labelFalselyPredictedElement.setText("Falsely Predicted")

    # buttonPredictedElement
    self.buttonPredictedElement = QtGui.QPushButton()
    self.buttonPredictedElement.setStyleSheet("background-color: rgb(255, 215, 80)")
    self.buttonPredictedElement.setEnabled(False)

    # labelPredictedElement
    self.labelPredictedElement = QtGui.QLabel()
    self.labelPredictedElement.setText("Predicted")

    # buttonActiveElement
    self.buttonActiveElement = QtGui.QPushButton()
    self.buttonActiveElement.setStyleSheet("background-color: rgb(50, 205, 50)")
    self.buttonActiveElement.setEnabled(False)

    # labelActiveElement
    self.labelActiveElement = QtGui.QLabel()
    self.labelActiveElement.setText("Active/Connected")

    # buttonLearningElement
    self.buttonLearningElement = QtGui.QPushButton()
    self.buttonLearningElement.setStyleSheet("background-color: rgb(125, 255, 0)")
    self.buttonLearningElement.setEnabled(False)

    # labelLearningElement
    self.labelLearningElement = QtGui.QLabel()
    self.labelLearningElement.setText("Learning")

    # buttonSelectedElement
    self.buttonSelectedElement = QtGui.QPushButton()
    self.buttonSelectedElement.setStyleSheet("background-color: rgb(0, 0, 255)")
    self.buttonSelectedElement.setEnabled(False)

    # labelSelectedElement
    self.labelSelectedElement = QtGui.QLabel()
    self.labelSelectedElement.setText("Selected")

    # layout
    layout = QtGui.QGridLayout()
    layout.addWidget(self.buttonInactiveElement, 0, 0)
    layout.addWidget(self.labelInactiveElement, 0, 1)
    layout.addWidget(self.buttonFalselyPredictedElement, 1, 0)
    layout.addWidget(self.labelFalselyPredictedElement, 1, 1)
    layout.addWidget(self.buttonPredictedElement, 2, 0)
    layout.addWidget(self.labelPredictedElement, 2, 1)
    layout.addWidget(self.buttonActiveElement, 3, 0)
    layout.addWidget(self.labelActiveElement, 3, 1)
    layout.addWidget(self.buttonLearningElement, 4, 0)
    layout.addWidget(self.labelLearningElement, 4, 1)
    layout.addWidget(self.buttonSelectedElement, 5, 0)
    layout.addWidget(self.labelSelectedElement, 5, 1)
    layout.setRowStretch(1, 100)

    # SimulationLegendForm
    self.setLayout(layout)
    self.setWindowTitle("Simulation Legend")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.setMinimumWidth(100)
    self.setMinimumHeight(150)

  #endregion



from nupic_studio.htm.segment import Segment, SegmentType

class Column:
  """
  A class only to group properties related to columns.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.x = -1
    """Position on X axis"""

    self.y = -1
    """Position on Y axis"""

    self.segment = Segment(SegmentType.proximal)
    """Proximal segment of this column"""

    self.cells = []
    """List of cells that compose this column."""

    #region 3d-tree properties (simulation form)

    self.tree3d_x = 0
    self.tree3d_y = 0
    self.tree3d_z = 0

    #endregion

    #endregion

  #endregion

  #region Methods

  def getCell(self, z):
    """
    Return the cell located at given position
    """

    for cell in self.cells:
      if cell.z == z:
        return cell

  def nextStep(self):
    """
    Perfoms actions related to time step progression.
    """

    self.segment.nextStep()
    for cell in self.cells:
      cell.nextStep()

  def calculateStatistics(self):
    """
    Calculate statistics after an iteration.
    """

    self.segment.calculateStatistics()
    for cell in self.cells:
      cell.calculateStatistics()

  #endregion



import os
import collections
import json
from nupic_studio.htm.node import NodeType
from nupic_studio.htm.link import Link
from nupic_studio.ui import Global

class Network:
  """
  The mains class for represent a HTM network composed by nodes and the links among them.
  """

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    #region Instance fields

    self.nodes = []
    """All regions/sensors in hierarchy which compose this network."""

    self.links = []
    """All links among all regions/sensors in hierarchy."""

    self.phases = []
    """Phases stores the processing order of one or more nodes.
    For example, in the Phase #0 all sensors should be processed.
    In turn, in the phase #1 all regions that are fed by the processed nodes in the previous phase also are processed.
    The last phase is the phase where top nodes (that do not feed nothing but only receive inputs) are processed."""

    #endregion

    #region Statistics properties

    self.statsPrecisionRate = 0.

    #endregion

  #endregion

  #region Methods

  def initialize(self):
    """
    Initialize this network.
    """

    # Initialize nodes using phases order
    for phaseNodes in self.phases:
      for node in phaseNodes:
        initialized = node.initialize()
        if not initialized:
          return

    return True

  def nextStep(self):
    """
    Perfoms actions related to time step progression.
    """

    # Process nodes using phases order
    for phaseNodes in self.phases:
      for node in phaseNodes:
        node.nextStep()

  def preparePhases(self):
    """
    Prepare phases analysing links between nodes.
    """

    self.phases = []

    # First put all sensors as non allocated nodes
    nonAllocatedNodes = []
    for node in self.nodes:
      if node.type == NodeType.sensor:
        nonAllocatedNodes.append(node)

    # Now find the regions that these sensors feed and process them recursively
    self.__processPhase(nonAllocatedNodes)

  def __processPhase(self, nonAllocatedNodes):

    # Check what nodes in the temporary list could be allocated into next phase
    # Only allocate a node to the next phase if this node not receives inputs of other non allocated node
    # if the node depends of other nodes, the latter ones has priority to be allocated into next phase
    phaseNodes = []
    for node in nonAllocatedNodes:
      canAllocate = True
      for link in self.links:
        if link.inNode == node and link.outNode in nonAllocatedNodes:
          canAllocate = False
          break
      if canAllocate:
        phaseNodes.append(node)
    self.phases.append(phaseNodes)

    # Remove allocated nodes from non allocated list
    for node in phaseNodes:
      if node in nonAllocatedNodes:
        nonAllocatedNodes.remove(node)

    # Add higher regions of the phase nodes into non allocated nodes list
    for node in phaseNodes:
      for link in self.links:
        if link.outNode == node and not link.inNode in nonAllocatedNodes:
          nonAllocatedNodes.append(link.inNode)

    # Process recursively the remaining nodes
    if len(nonAllocatedNodes) > 0:
      self.__processPhase(nonAllocatedNodes)

  def calculateStatistics(self):
    """
    Calculate statistics after an iteration.
    """

    # The network prediction precision is the average between all nodes precision
    precisionRate = 0.
    numNodes = 0
    for phaseNodes in self.phases:
      for node in phaseNodes:
        node.calculateStatistics()
        precisionRate += node.statsPrecisionRate
        numNodes += 1
    self.statsPrecisionRate = precisionRate / numNodes

  def addFeederNode(self, feederNode, fedNode):
    """
    Delete a node from hierarchy.
    """

    # Link the first node to the target node which it receive its output
    link = Link()
    link.outNode = feederNode
    link.inNode = fedNode

    # Add the node and the link to the network
    self.nodes.append(feederNode)
    self.links.append(link)

    self.preparePhases()

  def deleteFeederNode(self, node):
    """
    Delete a node from hierarchy.
    """

    # Delete first all nodes under the node
    for feeder in self.getFeederNodes(node):
      self.deleteFeederNode(feeder)

    # Remove all links involving this node
    for link in self.links:
      if link.outNode == node or link.inNode == node:
        self.links.remove(link)

    # Delete the node
    del node

    self.preparePhases()

  def getFeederNodes(self, node):
    """
    Delete a node from hierarchy.
    """

    # Get all nodes that feed the specified node
    feeders = []
    for link in self.links:
      if link.inNode == node:
        feeders.append(link.outNode)

    return feeders

  def getFedNodes(self, node):
    """
    Delete a node from hierarchy.
    """

    # Get all nodes that feed the specified node
    feds = []
    for link in self.links:
      if link.outNode == node:
        feds.append(link.inNode)

    return feds

  def getSourceCode(self):
    """
    Generate the source code of this network to help users to quickly integrate
    it into their applications using the NuPIC Network API.
    """

    code = ""

    # Generate instructions
    code += "'''\n"
    code += "The code below is automatically generated by NuPIC Studio and aims to help you to quickly integrate\n"
    code += "this HTM network into your application using the NuPIC Network API.\n"
    code += "For use this code, just create a new Python file and copy/paste these lines into it. Run it and voila!\n"
    code += "'''\n"
    code += "\n"

    # Generate 'import's
    code += "import json\n"
    code += "import copy\n"
    code += "from nupic.algorithms.anomaly import computeRawAnomalyScore\n"
    code += "from nupic.data.file_record_stream import FileRecordStream\n"
    code += "from nupic.engine import Network\n"
    code += "from nupic.encoders import MultiEncoder\n"
    code += "\n"

    # Generate 'network' creation
    code += "def createNetwork():\n"
    code += "\t'''\n"
    code += "\tCreate the Network instance.\n"
    code += "\t'''\n"
    code += "\n"
    code += "\tnetwork = Network()\n"
    code += "\n"

    # Generate 'nodes'
    for phaseNodes in reversed(self.phases):
      for node in phaseNodes:
        if node.type == NodeType.region:

          # Generate the spatial process params
          spParams = "{ "
          spParams += "\"spatialImp\": \"py\", "
          spParams += "\"columnCount\": " + str(node.width * node.height) + ", "
          spParams += "\"inputWidth\": " + str(node.getInputSize()) + ", "
          spParams += "\"potentialRadius\": " + str(node.potentialRadius) + ", "
          spParams += "\"potentialPct\": " + str(node.potentialPct) + ", "
          spParams += "\"globalInhibition\": " + str(int(node.globalInhibition)) + ", "
          spParams += "\"localAreaDensity\": " + str(node.localAreaDensity) + ", "
          spParams += "\"numActiveColumnsPerInhArea\": " + str(node.numActiveColumnsPerInhArea) + ", "
          spParams += "\"stimulusThreshold\": " + str(node.stimulusThreshold) + ", "
          spParams += "\"synPermInactiveDec\": " + str(node.proximalSynPermDecrement) + ", "
          spParams += "\"synPermActiveInc\": " + str(node.proximalSynPermIncrement) + ", "
          spParams += "\"synPermConnected\": " + str(node.proximalSynConnectedPerm) + ", "
          spParams += "\"minPctOverlapDutyCycle\": " + str(node.minPctOverlapDutyCycle) + ", "
          spParams += "\"minPctActiveDutyCycle\": " + str(node.minPctActiveDutyCycle) + ", "
          spParams += "\"dutyCyclePeriod\": " + str(node.dutyCyclePeriod) + ", "
          spParams += "\"maxBoost\": " + str(node.maxBoost) + ", "
          spParams += "\"seed\": " + str(node.spSeed) + ", "
          spParams += "\"spVerbosity\": 0"
          spParams += " }"

          # Generate the temporal process params
          tpParams = "{ "
          tpParams += "\"temporalImp\": \"py\", "
          tpParams += "\"columnCount\": " + str(node.width * node.height) + ", "
          tpParams += "\"inputWidth\": " + str(node.getInputSize()) + ", "
          tpParams += "\"cellsPerColumn\": " + str(node.numCellsPerColumn) + ", "
          tpParams += "\"initialPerm\": " + str(node.distalSynInitialPerm) + ", "
          tpParams += "\"connectedPerm\": " + str(node.distalSynConnectedPerm) + ", "
          tpParams += "\"minThreshold\": " + str(node.minThreshold) + ", "
          tpParams += "\"newSynapseCount\": " + str(node.maxNumNewSynapses) + ", "
          tpParams += "\"permanenceInc\": " + str(node.distalSynPermIncrement) + ", "
          tpParams += "\"permanenceDec\": " + str(node.distalSynPermDecrement) + ", "
          tpParams += "\"activationThreshold\": " + str(node.activationThreshold) + ", "
          tpParams += "\"seed\": " + str(node.tpSeed)
          tpParams += " }"

          code += "\t# Create '" + node.name + "' region\n"
          code += "\tcreateRegion(network=network, name='" + node.name + "', spParams=" + spParams + ", tpParams=" + tpParams + ")\n"

        elif node.type == NodeType.sensor:

          # Generate the sensor params
          sensorParams = "{ "
          sensorParams += "\"verbosity\": 0"
          sensorParams += " }"

          # If file name provided is a relative path, use project file path
          dataSource = ""
          if node.fileName != "":
            if os.path.dirname(node.fileName) == '':
              dataSource = os.path.dirname(Global.project.fileName) + '/' + node.fileName
            else:
              dataSource = node.fileName

          # Generate the encodings params
          encodingsParams = ""
          for encoding in node.encodings:
            name = encoding.encoderFieldName.split('.')[0]
            params = json.loads(encoding.encoderParams.replace("'", "\""), object_pairs_hook=collections.OrderedDict)

            encodingsParams = "\"" + name + "\": { "
            encodingsParams += "\"name\": \"" + name + "\", "
            encodingsParams += "\"fieldname\": \"" + encoding.dataSourceFieldName + "\", "

            for paramName in params:
              paramValue = params[paramName]
              if paramValue == "true":
                paramValue = 'True'
              elif paramValue == "false":
                paramValue = 'False'

              encodingsParams += "\"" + paramName + "\": " + str(paramValue) + ", "

            encodingsParams += "\"type\": \"" + encoding.encoderClass + "\""
            encodingsParams += " }, "

          # Generate 'sensor'
          code += "\t# Create '" + node.name + "' sensor\n"
          code += "\tcreateSensor(network=network, name='" + node.name + "', params=" + sensorParams + ", dataFile='" + dataSource + "', encodings={ " + encodingsParams + " })\n"

        for fed in self.getFedNodes(node):
          if node.type == NodeType.region:

            # Generate link between this region and the fed region
            code += "\tlinkRegionToRegion(network=network, outName='" + node.name + "', inName='" + fed.name + "')\n"

          elif node.type == NodeType.sensor:

            # Generate link between this sensor and the fed region
            code += "\tlinkSensorToRegion(network=network, outName='" + node.name + "', inName='" + fed.name + "')\n"

        code += "\n"

    code += "\treturn network\n"
    code += "\n"

    # Generate function for 'region' creation
    code += "def createRegion(network, name, spParams, tpParams):\n"
    code += "\t'''\n"
    code += "\tCreate a region given SP and TP parameters.\n"
    code += "\t'''\n"
    code += "\n"
    code += "\t# Create spatial node\n"
    code += "\tspNode = network.addRegion(name='sp' + name, nodeType='py.SPRegion', nodeParams=json.dumps(spParams)).getSelf()\n"
    code += "\tspNode.learningMode = True\n"
    code += "\tspNode.anomalyMode = False\n"
    code += "\n"
    code += "\t# Create temporal node\n"
    code += "\ttpNode = network.addRegion(name='tp' + name, nodeType='py.TPRegion', nodeParams=json.dumps(tpParams)).getSelf()\n"
    code += "\ttpNode.learningMode = True\n"
    code += "\ttpNode.inferenceMode = True\n"
    code += "\ttpNode.topDownMode = True\n"
    code += "\ttpNode.anomalyMode = True\n"
    code += "\n"
    code += "\t# Create link betwen SP and TP of the same region\n"
    code += "\tnetwork.link(srcName='sp' + name, destName='tp' + name, linkType='UniformLink', linkParams='')\n"
    code += "\tnetwork.link(srcName='tp' + name, destName='sp' + name, linkType='UniformLink', linkParams='', srcOutput='topDownOut', destInput='topDownIn')\n"
    code += "\n"

    # Generate function for 'sensor' creation
    code += "def createSensor(network, name, params, dataFile, encodings):\n"
    code += "\t'''\n"
    code += "\tCreate a sensor given its parameters.\n"
    code += "\t'''\n"
    code += "\n"
    code += "\t# Create database given file name\n"
    code += "\tdataSource = FileRecordStream(streamID=dataFile)\n"
    code += "\tdataSource.setAutoRewind(True)\n"
    code += "\n"
    code += "\t# Create multi-encoder to handle all sub-encoders\n"
    code += "\tencoder = MultiEncoder()\n"
    code += "\tencoder.addMultipleEncoders(fieldEncodings=encodings)\n"
    code += "\n"
    code += "\t# Create sensor node\n"
    code += "\tsensor = network.addRegion(name='sensor' + name, nodeType='py.RecordSensor', nodeParams=json.dumps(params)).getSelf()\n"
    code += "\tsensor.dataSource = dataSource\n"
    code += "\tsensor.encoder = encoder\n"
    code += "\n"

    # Generate function for link between regions function
    code += "def linkRegionToRegion(network, outName, inName):\n"
    code += "\t'''\n"
    code += "\tCreate links between a region and another higher one in the hierarchy.\n"
    code += "\t'''\n"
    code += "\n"
    code += "\t# Create link between TP from first region and SP of second region\n"
    code += "\tnetwork.link(srcName='tp' + outName, destName='sp' + inName, linkType='UniformLink', linkParams='')\n"
    code += "\n"

    # Generate function for link between sensor and region
    code += "def linkSensorToRegion(network, outName, inName):\n"
    code += "\t'''\n"
    code += "\tCreate links between sensor and region in the hierarchy.\n"
    code += "\t'''\n"
    code += "\n"
    code += "\t# Create link between Sensor and SP of the region\n"
    code += "\tnetwork.link(srcName='sensor' + outName, destName='sp' + inName, linkType='UniformLink', linkParams='')\n"
    code += "\tnetwork.link(srcName='sensor' + outName, destName='sp' + inName, linkType='UniformLink', linkParams='', srcOutput='resetOut', destInput='resetIn')\n"
    code += "\tnetwork.link(srcName='sp' + inName, destName='sensor' + outName, linkType='UniformLink', linkParams='', srcOutput='spatialTopDownOut', destInput='spatialTopDownIn')\n"
    code += "\tnetwork.link(srcName='sp' + inName, destName='sensor' + outName, linkType='UniformLink', linkParams='', srcOutput='temporalTopDownOut', destInput='temporalTopDownIn')\n"
    code += "\n"

    # Generate 'network' runing
    code += "def runNetwork(network, numIterations):\n"
    code += "\t'''\n"
    code += "\tRun the network up to n iterations.\n"
    code += "\t'''\n"
    code += "\n"

    # Generate 'network' initialization
    code += "\tnetwork.initialize()\n"
    code += "\n"

    # Generate list with sensors and regions that are fed by them
    code += "\t# Only encodings with 'EnableInference' turned 'ON' will be printed\n"
    code += "\tlinks = []\n"
    for node in self.nodes:
      if node.type == NodeType.sensor:
        sensorName = node.name
        regionName = self.getFedNodes(node)[0].name

        # Add to list only those encodings with inference enabled
        encodings = "["
        for encodingIdx in range(len(node.encodings)):
          encoding = node.encodings[encodingIdx]
          if encoding.enableInference:
            encodings += "['" + encoding.encoderFieldName + "', " + str(encodingIdx) + "], "
        encodings += "]"

        code += "\tlinks.append({ 'sensorName': '" + sensorName + "', 'regionName': '" + regionName + "', 'encodings': " + encodings + ", 'prevPredictedColumns': [] })\n"
    code += "\n"

    # Generate 'network' iteration
    code += "\t# Run the network showing current values from sensors and their anomaly scores\n"
    code += "\tprintRow('Iter', 'Sensor', 'Encoding', 'Current', 'Anomaly Score')\n"
    code += "\tfor i in range(numIterations):\n"
    code += "\t\tnetwork.run(1)\n"
    code += "\n"

    # Generate iteration rows
    code += "\t\tfor link in links:\n"
    code += "\t\t\tsensorName = link['sensorName']\n"
    code += "\t\t\tregionName = link['regionName']\n"
    code += "\t\t\tencodings = link['encodings']\n"
    code += "\t\t\tprevPredictedColumns = link['prevPredictedColumns']\n"
    code += "\n"
    code += "\t\t\tsensorNode = network.regions['sensor' + sensorName]\n"
    code += "\t\t\tspNode = network.regions['sp' + regionName]\n"
    code += "\t\t\ttpNode = network.regions['tp' + regionName]\n"
    code += "\n"
    code += "\t\t\t# The anomaly score is relation of active columns over previous predicted columns\n"
    code += "\t\t\tactiveColumns = spNode.getOutputData('bottomUpOut').nonzero()[0]\n"
    code += "\t\t\tanomalyScore = computeRawAnomalyScore(activeColumns, prevPredictedColumns)\n"
    code += "\n"
    code += "\t\t\tfor encoding in encodings:\n"
    code += "\t\t\t\tencodingFieldName = encoding[0]\n"
    code += "\t\t\t\tencodingFieldIdx = encoding[1]\n"
    code += "\n"
    code += "\t\t\t\t# Print the anomaly score along with the iteration number and current value of this encoding.\n"
    code += "\t\t\t\tcurrValue = sensorNode.getOutputData('sourceOut')[encodingFieldIdx]\n"
    code += "\t\t\t\tprintRow(i, sensorName, encodingFieldName, currValue, anomalyScore)\n"
    code += "\n"
    code += "\t\t\t# Store the predicted columns for the next iteration\n"
    code += "\t\t\tpredictedColumns = tpNode.getOutputData('topDownOut').nonzero()[0]\n"
    code += "\t\t\tlink['prevPredictedColumns'] = copy.deepcopy(predictedColumns)\n"
    code += "\n"
    code += "def printRow(iter, sensorName, encodingFieldName, currValue, anomalyScore):\n"
    code += "\t'''\n"
    code += "\tPrint a row with fixed-length fields to default output.\n"
    code += "\t'''\n"
    code += "\n"
    code += "\titer = str(iter).rjust(10) + ' '\n"
    code += "\tsensorName = str(sensorName).ljust(20) + ' '\n"
    code += "\tencodingFieldName = str(encodingFieldName).ljust(25) + ' '\n"
    code += "\tcurrValue = str(currValue).rjust(15) + ' '\n"
    code += "\tanomalyScore = str(anomalyScore).ljust(40)\n"
    code += "\tprint iter, sensorName, encodingFieldName, currValue, anomalyScore\n"
    code += "\n"

    # Generate 'main' method
    code += "if __name__ == '__main__':\n"
    code += "\tnetwork = createNetwork()\n"
    code += "\n"
    code += "\tnumIterations = input('Type the number of iterations: ')\n"
    code += "\trunNetwork(network, numIterations)\n"

    # Adjust special characters
    code = code.replace("\t", "  ")
    code = code.replace("\"", "[DOUBLE_QUOTE]") # Preserve double quotes before replace all single quotes to double quotes
    code = code.replace("'", "\"")
    code = code.replace("[DOUBLE_QUOTE]", "'")

    return code

  #endregion



import sys
import os
from setuptools import setup, find_packages

"""
This file should called to install the NuPIC Studio.
"""

REPO_DIR = os.getcwd()


def getPlatformInfo():
  """
  Identify platform
  """
  if "linux" in sys.platform:
    platform = "linux"
  elif "darwin" in sys.platform:
    platform = "darwin"
  elif "win32" in sys.platform:
    platform = "windows"
  else:
    raise Exception("Platform '%s' is unsupported!" % sys.platform)

  if sys.maxsize > 2**32:
    bitness = "64"
  else:
    bitness = "32"

  return platform, bitness


platform, bitness = getPlatformInfo()

# Get properties of the project like version, notes, etc
properties = {}
execfile(os.path.join(REPO_DIR, "nupic_studio", "__init__.py"), {}, properties)

# Call the setup process
os.chdir(REPO_DIR)
setup(
  name = 'nupic_studio',
  version = properties["__version__"],
  packages = find_packages(),
  package_data = {
    '': ['README.md', 'LICENSE'],
    'nupic_studio': ['nupic_studio.config'],
    'nupic_studio.images': ['*'],
    'nupic_studio.projects': ['*']},
  entry_points = {
    'gui_scripts': ['nupic_studio = nupic_studio.program:main']},
  description = 'NuPIC Studio is a virtual studio that allows developers to create, debug, and visualize HTM networks from NuPIC library',
  author='David Ragazzi',
  author_email='david_ragazzi@hotmail.com',
  url='https://github.com/nupic-community/nupic.studio',
  classifiers=[
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: OS Independent',
    'Development Status :: 5 - Production/Stable',
    'Environment :: Console',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering :: Artificial Intelligence'
  ],
  install_requires = [
    "PyOpenGL==3.1.0",
    "pyqtgraph==0.9.10"],
  long_description = """NuPIC Studio is a virtual studio that allows developers to create, debug, and visualize HTM networks from NuPIC library. Some of its advantages:
* Users can open, save, or change their "HTM projects" or of other developers. A typical project contains data to be trained, neural network configuration, statistics, etc, which can be shared to be analysed or integrated with other projects.
* Users can create their own encoders and sensors to feed the HTM network.
* Any changes in the nupic source can be immediatedly viewed. This helps users that wish test improvements like hierarchy, attention, and motor integration.
For more information, see numenta.org or the NuPIC wiki (https://github.com/nupic-community/nupic.studio/wiki)."""
)



import sys
from PyQt4 import QtGui, QtCore
from nupic_studio.ui import Global

class StartForm(QtGui.QDialog):

  #region Constructor

  def __init__(self):
    """
    Initializes a new instance of this class.
    """

    QtGui.QDialog.__init__(self)

    self.initUI()

  #endregion

  #region Methods

  def initUI(self):

    # buttonNew
    self.buttonNew = QtGui.QPushButton()
    self.buttonNew.setText("New Project")
    self.buttonNew.clicked.connect(self.__buttonNew_Click)

    # buttonOpen
    self.buttonOpen = QtGui.QPushButton()
    self.buttonOpen.setText("Open Project")
    self.buttonOpen.clicked.connect(self.__buttonOpen_Click)

    # buttonClose
    self.buttonClose = QtGui.QPushButton()
    self.buttonClose.setText("Close")
    self.buttonClose.clicked.connect(self.__buttonClose_Click)

    # formLayout
    layout = QtGui.QHBoxLayout()
    layout.addWidget(self.buttonNew)
    layout.addWidget(self.buttonOpen)
    layout.addWidget(self.buttonClose)

    # StartForm
    self.setLayout(layout)
    self.setWindowTitle("NuPIC Studio")
    self.setWindowIcon(QtGui.QIcon(Global.appPath + '/images/logo.ico'))
    self.resize(350, 50)

  #endregion

  #region Events

  def __buttonNew_Click(self, event):
    if Global.mainForm.newProject():
      Global.mainForm.showMaximized()
      self.close()

  def __buttonOpen_Click(self, event):
    if Global.mainForm.openProject():
      Global.mainForm.showMaximized()
      self.close()

  def __buttonClose_Click(self, event):
    sys.exit()


  #endregion
